/*
 * Raima Database Manager
 *
 * Copyright (C) 2016 Raims Inc.,  All rights reserved.
 *
 * Use of this software, whether in source code format, or in executable,
 * binary object code form, is governed by the Raima LICENSE which
 * is fully described in the LICENSE.TXT file, included within this
 * distribution of files.
 */

/** \file rdmbcdapi.h
 *  \brief Header for Binary-coded decimal (BCD) API
 */

#include "rdmbcdtypes.h"
#include "rdmretcodetypes.h"

#if defined(RDM_UNICODE)
#define rdm_bcdFromString RDM_T_FUNC (rdm_bcdFromString)
#define rdm_bcdToString RDM_T_FUNC (rdm_bcdToString)
#define rdm_bcdToStringWithScale RDM_T_FUNC (rdm_bcdToStringWithScale)
#endif

/** \defgroup rdm_bcd Binary-coded decimal (BCD) API
 * \ingroup datatypes
 *
 * The RDM BCD datatype API.  The functions here are located in \lib{base}
 *
 * \addtogroup rdm_bcd
 * @{
 */

#if defined(__cplusplus)
extern "C" {
#endif

/** \brief Add two numbers

    This function adds two numbers of the specified types.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdAdd (
    const RDM_BCD_T *operand1, /**< [in] First operand for the calculation */
    const RDM_BCD_T *operand2, /**< [in] Second operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Add two numbers

    This function adds two numbers of the specified types.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdAddDouble (
    const RDM_BCD_T *operand1, /**< [in] First operand for the calculation */
    double operand2,           /**< [in] Second operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Add two numbers

    This function adds two numbers of the specified types.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdAddInt32 (
    const RDM_BCD_T *operand1, /**< [in] First operand for the calculation */
    int32_t operand2,          /**< [in] Second operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);
/** \brief Add two numbers

    This function adds two numbers of the specified types.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdAddInt64 (
    const RDM_BCD_T *operand1, /**< [in] First operand for the calculation */
    int64_t operand2,          /**< [in] Second operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);
/** \brief Add two numbers

    This function adds two numbers of the specified types.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdAddUInt32 (
    const RDM_BCD_T *operand1, /**< [in] First operand for the calculation */
    uint32_t operand2,         /**< [in] Second operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);
/** \brief Add two numbers

    This function adds two numbers of the specified types.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdAddUInt64 (
    const RDM_BCD_T *operand1, /**< [in] First operand for the calculation */
    uint64_t operand2,         /**< [in] Second operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Adjust a BCD number

    This function adjusts a BCD number to the given precision and scale.

    \retval{sOKAY}
    */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdAdjust (
    RDM_BCD_T *argument, /**< [in/out] Argument to be adjusted */
    uint8_t precision, /**< [in] The precision to be used. The max precission is ::RDM_BCD_MAXPRECISION */
    uint8_t scale /**< [in] The scale to be used. The max scale is ::RDM_BCD_MAXSCALE */
);

/** \brief Take the ceiling of a number

    This function returns in \a result the nearest integer greater than or equal
   to \a operand.

    \retval{sOKAY}
    */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdCeiling (
    const RDM_BCD_T *operand, /**< [in] Operand for the calculation */
    RDM_BCD_T *result         /**< [out] Result of the calculation */
);

/** \brief Change the sign of a number

    This function changes the sign of a BCD number.

    \retval{sOKAY}
    */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdChgSign (
    const RDM_BCD_T *operand, /**< [in] Operand for the calculation */
    RDM_BCD_T *result         /**< [out] Result of the calculation */
);

/** \brief Compare two numbers

    This function compare two BCD numbers and return the result as an ::RDM_COMPARE.

    \returns ::RDM_COMPARE
    \retval{RDM_LT}
    \retval{RDM_EQ}
    \retval{RDM_GT}
 */
RDM_EXPORT RDM_COMPARE EXTERNAL_FCN rdm_bcdCompare (
    const RDM_BCD_T *operand1, /**< [in] First operand for the comparison */
    const RDM_BCD_T *operand2  /**< [in] Second operand for the comparison */
);

/** \brief Copy a BCD number

    This function makes a copy of a BCD number.

    \retval{sOKAY}
 */
RDM_EXPORT void EXTERNAL_FCN rdm_bcdCopy (
    const RDM_BCD_T *source, /**< [in] Source to be copied */
    RDM_BCD_T *result        /**< [out] Result copy */
);

/** \brief Take the cosine of a number

    This function takes the cosine of a BCD number.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdCos (
    const RDM_BCD_T *operand, /**< [in] Operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);
/** \brief Take the cotangent of a number

    This function takes the cotangent of a BCD number.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdCot (
    const RDM_BCD_T *operand, /**< [in] Operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Take the cosecant of a number

    This function takes the cosecant of a BCD number.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdCsc (
    const RDM_BCD_T *operand, /**< [in] Operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Divide two numbers

    This function divides two numbers of the specified types.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdDiv (
    const RDM_BCD_T *operand1, /**< [in] First operand for the calculation */
    const RDM_BCD_T *operand2, /**< [in] Second operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Divide two numbers

    This function divides two numbers of the specified types.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdDivByDouble (
    const RDM_BCD_T *operand1, /**< [in] First operand for the calculation */
    double operand2,           /**< [in] Second operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Divide two numbers

    This function divides two numbers of the specified types.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdDivByInt32 (
    const RDM_BCD_T *operand1, /**< [in] First operand for the calculation */
    int32_t operand2,          /**< [in] Second operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Divide two numbers

    This function divides two numbers of the specified types.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdDivByInt64 (
    const RDM_BCD_T *operand1, /**< [in] First operand for the calculation */
    int64_t operand2,          /**< [in] Second operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Divide two numbers

    This function divides two numbers of the specified types.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdDivByUInt32 (
    const RDM_BCD_T *operand1, /**< [in] First operand for the calculation */
    uint32_t operand2,         /**< [in] Second operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Divide two numbers

    This function divides two numbers of the specified types.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdDivByUInt64 (
    const RDM_BCD_T *operand1, /**< [in] First operand for the calculation */
    uint64_t operand2,         /**< [in] Second operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Divide two numbers

    This function divides two numbers of the specified types.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdDivDouble (
    double operand1,           /**< [in] First operand for the calculation */
    const RDM_BCD_T *operand2, /**< [in] Second operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Divide two numbers

    This function divides two numbers of the specified types.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdDivInt32 (
    int32_t operand1,          /**< [in] First operand for the calculation */
    const RDM_BCD_T *operand2, /**< [in] Second operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Divide two numbers

    This function divides two numbers of the specified types.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdDivInt64 (
    int64_t operand1,          /**< [in] First operand for the calculation */
    const RDM_BCD_T *operand2, /**< [in] Second operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Divide two numbers

    This function divides two numbers of the specified types.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdDivUInt32 (
    uint32_t operand1,         /**< [in] First operand for the calculation */
    const RDM_BCD_T *operand2, /**< [in] Second operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Divide two numbers

    This function divides two numbers of the specified types.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdDivUInt64 (
    uint64_t operand1,         /**< [in] First operand for the calculation */
    const RDM_BCD_T *operand2, /**< [in] Second operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Take the exponent of a number

    This function takes the exponent of a BCD number.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdExp (
    const RDM_BCD_T *operand, /**< [in] Operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Take the floor of a number

    This function returns in \a result the nearest integer less than or equal to \a operand.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdFloor (
    const RDM_BCD_T *operand, /**< [in] Operand for the calculation */
    RDM_BCD_T *result         /**< [out] Result of the calculation */
);

/** \brief Convert a double to a BCD number

    This function converts a double to BCD number.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdFromDouble (
    double operand,   /**< [in] Operand to be converted */
    RDM_BCD_T *result /**< [out] Result of the conversion */
);

/** \brief Convert a float to a BCD number

    This function converts a number given in a float argument to BCD number.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdFromFloat (
    float operand,    /**< [in] Operand to be converted */
    RDM_BCD_T *result /**< [out] Result of the conversion */
);

/** \brief Convert an int16_t to a BCD number

    This function converts an int16_t to a BCD number.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdFromInt16 (
    int16_t operand,  /**< [in] Operand to be converted */
    RDM_BCD_T *result /**< [out] Result of the conversion */
);

/** \brief Convert an int32_t to a BCD number

    This function converts an int32_t to a BCD number.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdFromInt32 (
    int32_t operand,  /**< [in] Operand to be converted */
    RDM_BCD_T *result /**< [out] Result of the conversion */
);

/** \brief Convert an int64_t to a BCD number

    This function converts an int64_t to a BCD number.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdFromInt64 (
    int64_t operand,  /**< [in] Operand to be converted */
    RDM_BCD_T *result /**< [out] Result of the conversion */
);

/** \brief Convert an int8_t to a BCD number

    This function converts an int8_t argument to BCD number.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdFromInt8 (
    int8_t operand,   /**< [in] Operand to be converted */
    RDM_BCD_T *result /**< [out] Result of the conversion */
);

/** \brief Convert a string to a BCD number

    This function converts a number given in a string argument to BCD number.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdFromString (
    const RDM_TCHAR_T *operand, /**< [in] Operand to be converted */
    RDM_BCD_T *result           /**< [out] Result of the conversion */
);

/** \brief Convert an uint16_t to a BCD number

    This function converts an uint16_t to a BCD number.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdFromUInt16 (
    uint16_t operand, /**< [in] Operand to be converted */
    RDM_BCD_T *result /**< [out] Result of the conversion */
);

/** \brief Convert an uint32_t to a BCD number

    This function converts an uint32_t to a BCD number.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdFromUInt32 (
    uint32_t operand, /**< [in] Operand to be converted */
    RDM_BCD_T *result /**< [out] Result of the conversion */
);

/** \brief Convert an uint64_t to a BCD number

    This function converts an uint64_t to a BCD number.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdFromUInt64 (
    uint64_t operand, /**< [in] Operand to be converted */
    RDM_BCD_T *result /**< [out] Result of the conversion */
);

/** \brief Convert an uint8_t to a BCD number

    This function converts an uint8_t to a BCD number.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdFromUInt8 (
    uint8_t operand,  /**< [in] Operand to be converted */
    RDM_BCD_T *result /**< [out] Result of the conversion */
);

/** \brief Is a number zero

    This function returns wether the BCD number is zero or not

    \returns RDM_BOOL_T
    \retval RDM_TRUE  The number is zero
    \retval RDM_FALSE The number is not zero
 */
RDM_EXPORT RDM_BOOL_T EXTERNAL_FCN rdm_bcdIsZero (
    const RDM_BCD_T *argument /**< [in] The argument that is checked */
);

/** \brief Take the natual logarithm of a number

    This function takes the natual logarithm of a BCD number.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdLog (
    const RDM_BCD_T *operand, /**< [in] Operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Take the logarithm base 10 of a number

    This function takes the logarithm base 10 of a BCD number.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdLog10 (
    const RDM_BCD_T *operand, /**< [in] Operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Return the maximum precision

    This function returns the maximum precision for a BCD number.

    \returns Max precision
    \retval RDM_BCD_MAXPRECISION
 */
RDM_EXPORT uint8_t EXTERNAL_FCN rdm_bcdMaxprec (void);

/** \brief Return the scale

    This function returns the scale for a BCD number.

    \returns Max scale
    \retval RDM_BCD_MAXSCALE
 */
RDM_EXPORT uint8_t EXTERNAL_FCN rdm_bcdMaxscale (void);

/** \brief Multiply two numbers

    This function multiplies two numbers of the specified types.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdMult (
    const RDM_BCD_T *operand1, /**< [in] First operand for the calculation */
    const RDM_BCD_T *operand2, /**< [in] Second operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Multiply two numbers

    This function multiplies two numbers of the specified types.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdMultDouble (
    const RDM_BCD_T *operand1, /**< [in] First operand for the calculation */
    double operand2,           /**< [in] Second operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Multiply two numbers

    This function multiplies two numbers of the specified types.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdMultInt32 (
    const RDM_BCD_T *operand1, /**< [in] First operand for the calculation */
    int32_t operand2,          /**< [in] Second operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Multiply two numbers

    This function multiplies two numbers of the specified types.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdMultInt64 (
    const RDM_BCD_T *operand1, /**< [in] First operand for the calculation */
    int64_t operand2,          /**< [in] Second operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Multiply two numbers

    This function multiplies two numbers of the specified types.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdMultUInt32 (
    const RDM_BCD_T *operand1, /**< [in] First operand for the calculation */
    uint32_t operand2,         /**< [in] Second operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Multiply two numbers

    This function multiplies two numbers of the specified types.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdMultUInt64 (
    const RDM_BCD_T *operand1, /**< [in] First operand for the calculation */
    uint64_t operand2,         /**< [in] Second operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Pi

    This function returns Pi as a BCD number.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdPi (
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *pi                        /***< [out] Pi result value */
);

/** \brief Take the power of a number

    This function takes the power of a BCD number.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdPow (
    const RDM_BCD_T *operand1, /**< [in] First operand for the calculation */
    const RDM_BCD_T *operand2, /**< [in] Second operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Take the power of a number

    This function takes the power of a BCD number.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdPowInt32 (
    const RDM_BCD_T *operand1, /**< [in] First operand for the calculation */
    int32_t operand2,          /**< [in] Second operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Take the power of a number

    This function takes the power of a BCD number.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdPowInt64 (
    const RDM_BCD_T *operand1, /**< [in] First operand for the calculation */
    int64_t operand2,          /**< [in] Second operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Take the power of a number

    This function takes the power of a BCD number.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdPowUInt32 (
    const RDM_BCD_T *operand1, /**< [in] First operand for the calculation */
    uint32_t operand2,         /**< [in] Second operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Take the power of a number

    This function takes the power of a BCD number.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdPowUInt64 (
    const RDM_BCD_T *operand1, /**< [in] First operand for the calculation */
    uint64_t operand2,         /**< [in] Second operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Take the reciprocal of a number

    This function takes the reciprocal of a BCD number.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdReciprocal (
    const RDM_BCD_T *operand, /**< [in] Operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Find the nearest integer of a BCD number

    This function rounds \a result up or down to the nearest integer from the
   input \a operand.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdRound (
    const RDM_BCD_T *operand, /**< [in] Operand for the calculation */
    RDM_BCD_T *result         /**< [out] Result of the calculation */
);

/** \brief Take the secant of a number

    This function takes the secant of a BCD number.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdSec (
    const RDM_BCD_T *operand, /**< [in] Operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Get the sign of a number

    This function returnes the sign of a BCD number.

    \returns a int16_t
    \retval 1 A positive number
    \retval 0 A number equal to 0
    \retval -1 A negative number
 */
RDM_EXPORT int16_t EXTERNAL_FCN rdm_bcdSign (
    const RDM_BCD_T *argument /**< [in] The argument that is checked */
);

/** \brief Take the sine of a number

    This function takes the sine of a BCD number.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdSin (
    const RDM_BCD_T *operand, /**< [in] Operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Take the square root of a number

    This function takes the square root of a BCD.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdSqrt (
    const RDM_BCD_T *operand, /**< [in] Operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Subtract two numbers

    This function subtracts two numbers of the specified types.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdSub (
    const RDM_BCD_T *operand1, /**< [in] First operand for the calculation */
    const RDM_BCD_T *operand2, /**< [in] Second operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Subtract two numbers

    This function subtracts two numbers of the specified types.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdSubDouble (
    const RDM_BCD_T *operand1, /**< [in] First operand for the calculation */
    double operand2,           /**< [in] Second operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Subtract two numbers

    This function subtracts two numbers of the specified types.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdSubFromDouble (
    double operand1,           /**< [in] First operand for the calculation */
    const RDM_BCD_T *operand2, /**< [in] Second operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Subtract two numbers

    This function subtracts two numbers of the specified types.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdSubFromInt32 (
    int32_t operand1,          /**< [in] First operand for the calculation */
    const RDM_BCD_T *operand2, /**< [in] Second operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Subtract two numbers

    This function subtracts two numbers of the specified types.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdSubFromInt64 (
    int64_t operand1,          /**< [in] First operand for the calculation */
    const RDM_BCD_T *operand2, /**< [in] Second operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Subtract two numbers

    This function subtracts two numbers of the specified types.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdSubFromUInt32 (
    uint32_t operand1,         /**< [in] First operand for the calculation */
    const RDM_BCD_T *operand2, /**< [in] Second operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Subtract two numbers

    This function subtracts two numbers of the specified types.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdSubFromUInt64 (
    uint64_t operand1,         /**< [in] First operand for the calculation */
    const RDM_BCD_T *operand2, /**< [in] Second operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Subtract two numbers

    This function subtracts two numbers of the specified types.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdSubInt32 (
    const RDM_BCD_T *operand1, /**< [in] First operand for the calculation */
    int32_t operand2,          /**< [in] Second operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Subtract two numbers

    This function subtracts two numbers of the specified types.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdSubInt64 (
    const RDM_BCD_T *operand1, /**< [in] First operand for the calculation */
    int64_t operand2,          /**< [in] Second operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Subtract two numbers

    This function subtracts two numbers of the specified types.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdSubUInt32 (
    const RDM_BCD_T *operand1, /**< [in] First operand for the calculation */
    uint32_t operand2,         /**< [in] Second operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Subtract two numbers

    This function subtracts two numbers of the specified types.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdSubUInt64 (
    const RDM_BCD_T *operand1, /**< [in] First operand for the calculation */
    uint64_t operand2,         /**< [in] Second operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Take the tangent of a number

    This function takes the tangent of a BCD number.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdTan (
    const RDM_BCD_T *operand, /**< [in] Operand for the calculation */
    RDM_BCD_ROUNDING_MODE rounding_mode, /**< [in] rounding mode */
    RDM_BCD_T *result                    /**< [out] Result of the calculation */
);

/** \brief Convert a BCD number to a double

    This function converts a BCD number to a double.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdToDouble (
    const RDM_BCD_T *operand, /**< [in] Operand to be converted */
    double *result            /**< [out] Result of the conversion */
);

/** \brief Convert a BCD number to a float

    This function converts a BCD number to a float.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdToFloat (
    const RDM_BCD_T *operand, /**< [in] Operand to be converted */
    float *result             /**< [out] Result of the conversion */
);

/** \brief Convert a BCD number to an int16_t

    This function converts a BCD number to an int16_t.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdToInt16 (
    const RDM_BCD_T *operand, /**< [in] Operand to be converted */
    int16_t *result           /**< [out] Result of the conversion */
);

/** \brief Convert a BCD number to an int32_t

    This function converts a BCD number to an int32_t.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdToInt32 (
    const RDM_BCD_T *operand, /**< [in] Operand to be converted */
    int32_t *result           /**< [out] Result of the conversion */
);

/** \brief Convert a BCD number to an int64_t

    This function converts a BCD number to an int64_t.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdToInt64 (
    const RDM_BCD_T *operand, /**< [in] Operand to be converted */
    int64_t *result           /**< [out] Result of the conversion */
);

/** \brief Convert a BCD number to an int8_t

    This function converts a BCD number to an int8_t.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdToInt8 (
    const RDM_BCD_T *operand, /**< [in] Operand to be converted */
    int8_t *result            /**< [out] Result of the conversion */
);

/** \brief Convert a BCD number to a string

    This function converts a BCD number to a string.

    The number of bytes needed for the conversion can be retrieved
    through \par bytesOut by setting result to NULL.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdToString (
    const RDM_BCD_T *operand, /**< [in] Operand to be converted */
    RDM_BOOL_T useExponent,   /**< [IN] Use exponent in output */
    RDM_TCHAR_T *result,      /**< [out] Result of the conversion */
    size_t inBytes,           /**< [in] Length of result buffer in bytes */
    size_t *bytesOut /**< [out] A pointer to the number of bytes returned (can be NULL) */
);

/** \brief Convert a BCD number with scale to a string

    This function converts a BCD number with scale to a string.

    The number of bytes needed for the conversion can be retrieved
    through \par bytesOut by setting result to NULL.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdToStringWithScale (
    const RDM_BCD_T *operand, /**< [in] Operand to be converted */
    uint8_t scale,            /**< [in] Scale to use for the output */
    RDM_BOOL_T useExponent,   /**< [IN] Use exponent in output */
    RDM_TCHAR_T *result,      /**< [out] Result of the conversion */
    size_t inBytes,           /**< [in] Length of result buffer in bytes */
    size_t *bytesOut /**< [out] A pointer to the number of bytes returned (can be NULL) */
);

/** \brief Convert a BCD number to an uint16_t

    This function converts a BCD number to an uint16_t.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdToUInt16 (
    const RDM_BCD_T *operand, /**< [in] Operand to be converted */
    uint16_t *result          /**< [out] Result of the conversion */
);

/** \brief Convert a BCD number to an uint32_t

    This function converts a BCD number to an uint32_t.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdToUInt32 (
    const RDM_BCD_T *operand, /**< [in] Operand to be converted */
    uint32_t *result          /**< [out] Result of the conversion */
);

/** \brief Convert a BCD number to an uint64_t

    This function converts a BCD number to an uint64_t.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdToUInt64 (
    const RDM_BCD_T *operand, /**< [in] Operand to be converted */
    uint64_t *result          /**< [out] Result of the conversion */
);

/** \brief Convert a BCD number to an uint8_t

    This function converts a BCD number to an uint8_t.

    \retval sOKAY Success
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_bcdToUInt8 (
    const RDM_BCD_T *operand, /**< [in] Operand to be converted */
    uint8_t *result           /**< [out] Result of the conversion */
);

#if defined(__cplusplus)
}
#endif

/** @}*/
