/* ----------------------------------------------------------------------------
 * Raima Database Manager
 *
 * Copyright (c) 2014 Raima Inc.,  All rights reserved.
 *
 * Use of this software, whether in source code format, or in executable,
 * binary object code form, is governed by the Raima LICENSE which
 * is fully described in the LICENSE.TXT file, included within this
 * distribution of files.
 * ----------------------------------------------------------------------------
 */

/* PLEASE READ AND UNDERSTAND PRIOR TO MODIFYING THIS FILE
 *
 * This header file should contain the declarations for functions that are
 * exported from the RDM runtime libarary as part of the published
 * RDM_DB interface.  These functions will be availabe for use by customers
 * and other RDM modules, therefore they should all be declared with RDM_EXPORT
 * and EXTERNAL_FCN.
 *
 * If the API takes a string as a parameter it should always use RDM_TCHAR_T
 * and use a RDM_T_FUNC wrapper.
 *
 * This header file should only contain function definitions.  Types,
 * structures, enums etc. required for the RDM_DB interface (a parameter to one
 * of the public APIs), should be declared in include/rdmtypes.h
 *
 */

/** \file rdmdbapi.h
 *  \brief Header for the RDM Database APIs
 */

#include "rdmtypes.h"
#include "rdmrowidtypes.h"
#include "rdmtfstypes.h"

#if defined(__cplusplus)
extern "C" {
#endif

/**
 *
 * \defgroup db RDM Database APIs
 * \ingroup tfs
 *
 *  The core database API.  Most of the functions here are located in \lib{rdm}
 *
 *  A database handle can be instantiated using rdm_rdmAllocDatabase() or
 * rdm_tfsAllocDatabase().
 *
 * \addtogroup db
 * @{
 * \defgroup db_initialization Initialization Functions
 *
 *    Initialization functions for the core database API.  All the
 *    fuctions here except for rdm_dbTriggersOn() are located in
 *    \lib{rdm}
 *
 * \defgroup db_replication Database Replication Functions
 *
 *    Replication functions for the core database API.  All the
 *    fuctions here are located in \lib{rdm}
 *
 * \defgroup db_locking Locking and Transaction Start Functions
 *
 *    Locking and Transaction functions for the core database API.  All the
 *    fuctions here are located in \lib{rdm}
 *
 *  \defgroup db_cleanup Cleanup Functions
 *
 *    Cleanup API for the core database.  All the fuctions here are
 *    located in \lib{rdm}
 *
 *  \defgroup db_cursor_init Cursor Initialization Functions
 *
 *    Cursor initialization API for the core database.  All the
 *    fuctions here are located in \lib{rdm}
 *
 *    See \ref cursor for a more detailed description of a cursor.
 *
 *  \defgroup db_association_rowid Cursor Association Functions (rowscan)
 *
 *    Cursor association API for retrieving rows in rowid order.
 *    All the fuctions here are located in \lib{rdm}
 *
 *    See \ref cursor for a more detailed description of a cursor.
 *
 *  \defgroup db_association_key Cursor Association Functions (keyscan)
 *
 *    Cursor association API for retrieving rows in key order.  All
 *    the fuctions here are located in \lib{rdm}
 *
 *    See \ref cursor for a more detailed description of a cursor.
 *
 *  \defgroup db_insert Insert Functions
 *
 *    Cursor association API for inserting rows.  All the fuctions
 *    here are located in \lib{rdm}
 *
 *  \defgroup db_update Update Functions
 *
 *    Schema API for altering the schema.  All the fuctions here are
 *    located in \lib{rdm}
 *
 *  \defgroup db_delete Delete Functions
 *
 *    API for deleting all rows from a table or a database.  All the
 *    functions here are located in \lib{rdm}
 *
 *  \defgroup db_information Database Information Functions
 *
 *    API for retrieving the TFS information for a database.  All
 *    the fuctions here are located in \lib{rdm}
 *
 *  \defgroup db_encryption Encryption Functions
 *
 *    Encryption API for a database.  All the fuctions here are
 *    located in \lib{rdm}
 *
 *    Additional API for instantiating an encryption context consult
 *    \ref rdm_encryption.
 *
 *
 *  \defgroup db_error Error handling functions
 *
 *    Error handling API for a database.  All the fuctions here are
 *    located in \lib{rdm}
 *
 *  \defgroup db_utilities Utility functions
 *
 *    Utility API for a database.  Unless otherwise stated functions
 *    here are located in \lib{rdm}
 *
 *  \defgroup db_miscellaneous Miscellaneous functions
 *
 *    Miscellaneous API.  All the fuctions here are
 *    located in \lib{rdm}
 *
 *  @}*/

#if defined(RDM_UNICODE)
#define rdm_dbAlterCatalog RDM_T_FUNC (rdm_dbAlterCatalog)
#define rdm_dbEncrypt RDM_T_FUNC (rdm_dbEncrypt)
#define rdm_dbExport RDM_T_FUNC (rdm_dbExport)
#define rdm_dbGetOption RDM_T_FUNC (rdm_dbGetOption)
#define rdm_dbGetOptions RDM_T_FUNC (rdm_dbGetOptions)
#define rdm_dbGetInfo RDM_T_FUNC (rdm_dbGetInfo)
#define rdm_dbImport RDM_T_FUNC (rdm_dbImport)
#define rdm_dbOpen RDM_T_FUNC (rdm_dbOpen)
#define rdm_dbSetCatalog RDM_T_FUNC (rdm_dbSetCatalog)
#define rdm_dbSetCatalogFromFile RDM_T_FUNC (rdm_dbSetCatalogFromFile)
#define rdm_dbCompileCatalog RDM_T_FUNC (rdm_dbCompileCatalog)
#define rdm_dbCompileCatalogFromFile RDM_T_FUNC (rdm_dbCompileCatalogFromFile)
#define rdm_dbLoadCatalog RDM_T_FUNC (rdm_dbLoadCatalog)
#define rdm_dbLoadCatalogFromFile RDM_T_FUNC (rdm_dbLoadCatalogFromFile)
#define rdm_dbSetErrorHandler RDM_T_FUNC (rdm_dbSetErrorHandler)
#define rdm_dbSetOption RDM_T_FUNC (rdm_dbSetOption)
#define rdm_dbSetOptions RDM_T_FUNC (rdm_dbSetOptions)
#define rdm_dbVacuum RDM_T_FUNC (rdm_dbVacuum)
#endif

/** \brief Allocate a cursor

    This function allocates and initializes a cursor. The new cursor is
    associated with the specified database. Once a cursor has been initialized
    it can be used in other ::RDM_CURSOR function calls.

    An allocated cursor is not associated with any rows. One of the association
    functions (\c{rdm_dbGet*}, \c{rdm_cursorGet*}, and \c{rdm_dbInsert*}) need
   to be called to associate the cursor with a collection of rows. Most of the
   other cursor APIs will return an error if called with a cursor that is
   allocated but not yet associated with rows.

    It is not necessary to explicitly allocate a cursor. The association
    functions will automatically allocate a cursor if the \a pCursor parameter
    is set to NULL prior to calling the association function.

    \par Locking Requirements
    None

    \ingroup db_cursor_init

    \retval{sOKAY}
    \retval{eDBNOTOPEN}
    \retval{eCURSORDB}
    \retval{ePRECOMMITTED}

 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbAllocCursor (
    RDM_DB db,          /**< [in]  A valid RDM database handle */
    RDM_CURSOR *pCursor /**< [out] The cursor to be allocated */
);

/** \brief Process a schema DDL command

    This function processes a schema DDL (data definition language) command

    The DDL commands allow for modifying the format and structure of a database.
    This function can be used to create a new database or to alter the structure
    of an existing database. The use of this function requires exclusive access
    to the schema. This can be gained by issuing a call to
    rdm_dbStartUpdate() with the schema (::RDM_LOCK_SCHEMA) as one of the
    tables to write lock.

    The changes made to the schema will be visible locally immediately, but will
    not be made available to other sessions until the transaction has been
    committed. If the transaction is aborted then the database structure will
    be unchanged.

    \par Locking Requirements
    Write lock on the schema (::RDM_LOCK_SCHEMA)

    \ingroup db_update

    \retval{sOKAY}
    \retval{eNOSTARTUPDATE}
    \retval{eNOTLOCKED}
    \retval{eDDL_ALTERACT}
    \retval{eDDL_CONSTRNOTFOUND}
    \retval{eDDL_DROPNOTNULL}
    \retval{eDDL_DUPTABNAME}
    \retval{eDDL_NULLCONFLICT}
    \retval{eDDL_SYNTAX}
    \retval{eINVFORUNION}
    \retval{ePRECOMMITTED}
    \retval{eREADONLY}

    \see rdm_dbStartUpdate
    \see rdm_dbSetCatalog
    \see rdm_dbEnd

    \see ddl_ug
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbAlterCatalog (
    RDM_DB db,                 /**< [in] A valid RDM database handle */
    const RDM_TCHAR_T *ddlStmt /**< [in] The ddl statement to execute */
);

RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbClearCache (
    RDM_DB db /**< [in] A valid RDM database handle */
);

/** \brief Close the database associated with a database handle

    This function will close the database associated with the specified database
    handle. If the database has a transaction active the transaction will be
    aborted. If there are any cursors associated with the database they will be
    automatically freed and no longer valid.

    After the database is closed the handle can be reused by associating it
    with another database through one of the database open calls. If you are
    finished with the database handle you can should call rdm_dbFree() to
    free the resources associated with the handle.

    \retval{sOKAY}
    \retval{eDBNOTOPEN}
    \retval{sTRABORTED}

    \see rdm_dbOpen
    \see rdm_dbFree

    \ingroup db_cleanup
    */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbClose (
    RDM_DB db /**< [in] A valid RDM database handle */
);

/** \brief Remove all rows from a database

    This function removes all rows from all tables in the database.

    No triggers or other checks will be performed when a database has all of
    it's rows delete in this manner. Once this is called the database will be
    completely empty with no rows in any of the database tables.

    This function must be performed within a transaction (optional in exclusive
    mode or when used with the standalone transactional file server). If the
    transaction is aborted, the database will remain unchanged.

    Please note that any row IDs that was used prior to this call
    will be reused after a successful call to this function.

    \par Locking Requirements
    Write lock on all database tables (\ref RDM_LOCK_ALL).

    \retval{sOKAY}
    \retval{eDBNOTOPEN}
    \retval{eNOSTARTUPDATE}
    \retval{eNOTLOCKED}
    \retval{ePRECOMMITTED}
    \retval{eREADONLY}

    \see rdm_dbStartUpdate
    \see rdm_cursorDeleteRow
    \see rdm_dbDeleteAllRowsFromTable
    \see rdm_dbEnd

    \ingroup db_delete
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbDeleteAllRowsFromDatabase (
    RDM_DB db /**< [in] A valid RDM database handle */
);

/** \brief Remove all rows from a table

    This function deletes all of the rows in the specified table.

    The ability to delete rows will be based on the triggered action specified
    in the table definition. Rows that only references other rows will be
    deleted and the references automatically removed. The behavior of a row
    that is referenced by other rows will be dependent on the \reserved{on
   delete} triggered action.

    If the triggered action is:
    - \reserved{restrict} -- an error code will be returned and no rows will be
   deleted
    - \reserved{set null} -- all of rows that reference the table to be
   initialized will have their referencing foreign key columns set to NULL.
    - \reserved{cascade} -- all of the rows that reference the table to be
   initialized will also be deleted. If there are any referential integrity
   issues with the referencing rows an error code will be returned.

    \par Locking Requirements
    Write lock on the table associated with \a tableId and read locks on
    tables with foreign keys referencing this table.

    \retval{sOKAY}
    \retval{eDBNOTOPEN}
    \retval{eDBOPEN}
    \retval{eHASMEMBERS}
    \retval{eNOSTARTUPDATE}
    \retval{eNOTLOCKED}
    \retval{eREADONLY}

    \see rdm_dbStartUpdate
    \see rdm_cursorDeleteRow
    \see rdm_dbDeleteAllRowsFromDatabase
    \see rdm_dbEnd

    \ingroup db_delete
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbDeleteAllRowsFromTable (
    RDM_DB db, /**< [in] A valid RDM database handle */
    RDM_TABLE_ID
        tableId /**< [in] The table identifier for the table to be initialized */
);

/** \brief Get memory usage information for a database

    This function retrieves memory usage information for a database

    The memory usage is split between system memory which is memory that is used
    by the engine and user data which is memory used for storing the user data
    (rows, keys, blobs, schema, etc.).

    For a disk based database the user memory usage will always be 0 as the data
   is stored on disk and not in-memory.

    Both disk and in-memory databases require system memory.

    \par Locking Requirements
    None

    \retval{sOKAY}
    \retval{eDBNOTOPEN}


    \ingroup db_information
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbGetMemoryUsage (
    RDM_DB db, /**< [in] A valid RDM database handle */
    uint64_t *
        systemCurr, /**< [out] The current amount of system memory in use for
                       the database (does not include user data - rows, keys, etc.) */
    uint64_t *systemMax, /**< [out] The maximum total amount of system memory in
                            use for the database (does not include user data - rows, keys, etc.) */
    uint64_t *userCurr, /**< [out] The current amount of memory for user data in the database */
    uint64_t *userMax /**< [out] The maximum amount of memory for user data in the database */
);

/** \brief Encrypt a database

    This function can encrypt, decrypt, and re-encrypt an existing RDM database.

    When re-encrypting a database image you can choose whether to do a full or
    quick re-encryption. A quick re-encryption will just change the passcode
    used for encrypting the randomly generated key used to encrypt the actual
    database contents. In this case the database contents do not need to be
    read and rewritten so the operation can be performed very quickly.

    A full re-encryption will generate a new internal encryption key and will
    require each database page to be read, decrypted, re-encrypted, and
    rewritten to the file. This can be a lengthy operation for very large
    database images.

    To decrypt an encrypted database pass in a NULL encryption context (enc).

    \par Locking Requirements
    This function requires that the database be opened in exclusive mode.

    \retval{sOKAY}
    \retval{eDBNOTOPEN}
    \retval{eEXCLUSIVE}
    \retval{eTRACTIVE}

    \see rdm_dbOpen
    \see rdm_tfsAllocEncrypt
    \see rdm_dbGetEncrypt

    \ingroup db_encryption
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbEncrypt (
    RDM_DB db,       /**< [in] A valid RDM database handle */
    RDM_ENCRYPT enc, /**< [in] The new encryption context (can be NULL to decrypt) */
    const RDM_TCHAR_T *optString /**< [in] A semicolon delimited string that
                                    contains the utility options */
);

/** \brief End a transactional operation

    rdm_dbEnd() will commit a transactional operation for the specified
   database. If any updates have been made to the database then those will be
   committed and all locks will be freed.

    If a snapshot is active then it will be released.

    If there are only read locks active they will be freed.

    If there are any active transaction handles they will be invalidated and
    should not be used.

    See \ref locking for a more detailed description.

    \par Locking Requirements
    None

    \retval{sOKAY}

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbEndRollback
    \see rdm_dbPrecommit
    \see rdm_transEnd
    \see rdm_dbClose

    \ingroup db_locking
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbEnd (
    RDM_DB db /**< [in] A valid RDM database handle */
);

/** \brief End and rollback a transactional operation

    rdm_dbEndRollback() will end a transactional operation with a rollback
    for the specified database.

    If any updates have been made to the database then those will be rolled
    back and all locks will be freed.

    If a snapshot is active then it will be released.

    If there are only read locks active they will be freed.

    If there are any active transaction handles they will be invalidated and
    should not be used.

    See \ref locking for a more detailed description.

    \par Locking Requirements
    None

    \retval{sOKAY}
    \retval{ePRECOMMITTED}

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbPrecommit
    \see rdm_dbEnd
    \see rdm_dbClose

    \ingroup db_locking
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbEndRollback (
    RDM_DB db /**< [in] A valid RDM database handle */
);

/** \brief Database export API

    This API exports data from the database to ASCII text files  or an XML file
    that can then be moved to another computer, or used as input into other
    tools that read the formatted files (for example: rdm_dbImport()).

    For CSV export the rdm_dbExport() API writes all data from each table to
    a separate file; it is the table name with file extension \c{.txt}. If a
   table name, is specified, then only that table's rows are exported. The
   created files will consist of one text line per row occurrence with a
   separator characters separating the columns.

    The CSV export has two ways of printing extended ASCII characters to the
    output files. The default is to write out the octal value. Since
    rdm_dbImport() cannot handle this method, the extended option can be
    used to print the extended ASCII character as the character. For example,
    ASCII 2 would be printed as "\002" without the extended and as a smiley face
    with the extended option.

    For XML exports the rdm_dbExport() API writes all data from the database
    to an XML file. By exporting data into XML format, consumers of the data can
    be RDM (e.g. for database archiving), RDM Server, or other XML consumers,
    including other database products such as ADO.NET. The XML content created
    by the rdm_dbExport() API will allow reconstruction of sets (in the case
    of RDM as the consumer), or interpretation by any normal XML consumer.

    This function is located in \lib{export}

    \par Locking Requirements
    Read locks on the tables being exported.

    \retval{sOKAY}
    \retval{eDBNOTOPEN}

    \ingroup db_utilities

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbImport
    \see rdm_dbPersistInMemory
    \see rdm_dbEnd
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbExport (
    RDM_DB db,                   /**< [in] A valid RDM database handle */
    const RDM_TCHAR_T *optString /**< [in] A semicolon delimited string that
                                    contains the utility options */
);

/** \brief Database vacuum API

    This API vacuums delete data from the disk-based database pack files.

    This function is located in \lib{rdm}

    \par Locking Requirements
    None

    \retval{sOKAY}
    \retval{eDBNOTOPEN}

    \see rdm_dbSetOptions
    \see rdm_dbFlushIdIndex
    \see rdm_dbCreateNewPackFile
    \see \rdmvacuum

    \ingroup db_utilities
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbVacuum (
    RDM_DB db,                   /**< [in] A valid RDM database handle */
    const RDM_TCHAR_T *optString /**< [in] A semicolon delimited string that
                                    contains the utility options */
);

/** \brief Flush the database ID Index

    This API forces all changes to a databases ID Index to be flushed to disk.
    Typically the ID Index is flushed when a number of changes have been made,
    but this can force the ID Index to be flushed immediately.

    This function is located in \lib{rdm}

    \par Locking Requirements
    None

    \retval{sOKAY}
    \retval{eDBNOTOPEN}

    \see rdm_dbSetOptions
    \see rdm_dbCreateNewPackFile
    \see rdm_dbVacuum
    \see \rdmvacuum

    \ingroup db_utilities
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbFlushIdIndex (
    RDM_DB db /**< [in] A valid RDM database handle */
);

/** \brief Rebuild the database ID Index

    This API forces a complete rebuild of the ID Index of an on-disk database.
    It is a no-op for an in-memory database.

    This function is located in \lib{rdm}

    \par Locking Requirements
    None

    \retval{sOKAY}
    \retval{eDBNOTOPEN}

    \see rdm_dbSetOptions
    \see rdm_dbCreateNewPackFile
    \see rdm_dbVacuum
    \see \rdmvacuum

    \ingroup db_utilities
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbRebuildIdIndex (
    RDM_DB db /**< [in] A valid RDM database handle */
);

/** \brief Create a new pack file

    This API forces the database to create a new pack file.  All changes will
   now be written to the new pack.  A new pack is typically created when the
   current pack reaches the max-pack-file-size threshold.  This function forces
   a new pack independent of the size of the current pack.

    This function is located in \lib{rdm}

    \par Locking Requirements
    None

    \retval{sOKAY}
    \retval{eDBNOTOPEN}

    \see rdm_dbSetOptions
    \see rdm_dbFlushIdIndex
    \see rdm_dbVacuum
    \see \rdmvacuum

    \ingroup db_utilities
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbCreateNewPackFile (
    RDM_DB db /**< [in] A valid RDM database handle */
);

/** \brief Free a database handle

    This function will free a database handle. If the database handle has a
    database open then the database will be automatically closed prior to
    freeing the handle. The rules for closing a database apply when you are
    directly freeing a handle. If the database has a transaction active the
    transaction will be aborted. If there are any cursors associated with the
    database they will be automatically freed and no longer valid.

    After the handle has been freed it can no longer be used.

    \retval{sOKAY}

    \ingroup db_cleanup

    \see rdm_tfsAllocDatabase
    \see rdm_dbClose
    \see rdm_cursorFree
    \see rdm_tfsFree
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbFree (
    RDM_DB db /**< [in] A valid RDM database handle */
);

/** \brief Get the encryption context associated with a database handle

    This function allows the application to retrive the encryption context
    associated with a database handle. It is possible that multiple encryption
    contexts are associated with a database handle. If that is the case this
    function will return the most recently associated encryption context.

    \par Locking Requirements
    None

    \retval{sOKAY}

    \see rdm_tfsAllocDatabase
    \see rdm_dbSetEncrypt
    \see rdm_tfsGetEncrypt

    \ingroup db_encryption
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbGetEncrypt (
    RDM_DB db,       /**< [in] A valid RDM database handle */
    RDM_ENCRYPT *enc /**< [in] A pointer to an encryption context */
);

/** \brief Get the RDM_DB handle's current lock status of a table

    This function returns the current lock status of the specified table.
    If the specified ::RDM_DB handle does not have a lock on the table, then the
   status will be ::RDM_LOCK_FREE regardless of whether any other sessions have
   the table locked. This function allows the user to know if the specified
   ::RDM_DB handle (\a db) has access to the table.

    \note This function cannot be used to determine if another ::RDM_DB handle
   (e.g., another session) has the specified table locked.

    ::RDM_LOCK_STATUS values that can be returned in \a status are:

    | Status                | Description                     |
    |:----------------------|:--------------------------------|
    | ::RDM_LOCK_FREE       | \copydetails RDM_LOCK_FREE      |
    | ::RDM_LOCK_READ       | \copydetails RDM_LOCK_READ      |
    | ::RDM_LOCK_WRITE      | \copydetails RDM_LOCK_WRITE     |
    | ::RDM_LOCK_SNAPSHOT   | \copydetails RDM_LOCK_SNAPSHOT  |
    | ::RDM_LOCK_EXCLUSIVE  | \copydetails RDM_LOCK_EXCLUSIVE |

    \par Locking Requirements
    None

    \retval{sOKAY}
    \retval{eDBNOTOPEN}

    \see rdm_dbStartUpdate
    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_cursorGetLockStatus
    \see rdm_dbGetTransactionStatus
    \see rdm_dbEnd
    \see rdm_transEnd

    \ingroup db_locking
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbGetLockStatus (
    RDM_DB db,              /**< [in] A valid RDM database handle */
    RDM_TABLE_ID tableId,   /**< [in] The table identifier */
    RDM_LOCK_STATUS *status /**< [out] The current lock status for the
                              specified table */
);

/** \brief Get a single database option into a string

    This function retrieves a single database option into a string value. The
    \a keyword argument accepts the "key" for the function to retreive into the
    buffer supplied by the \a optValue argument.

    The \a bytesIn argument is the length of the \a optValue buffer.

    \includedoc dox/sqldb-options.mdinc

    \retval{sOKAY}
    \retval{eINVOPTION}
    \retval{eINVOPTIONVALUE}

    \see rdm_tfsAllocDatabase
    \see rdm_dbOpen
    \see rdm_dbGetOptions
    \see rdm_dbSetOption
    \see rdm_dbFree

    \ingroup db_initialization
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbGetOption (
    RDM_DB db,                  /**< [in] A valid RDM database handle */
    const RDM_TCHAR_T *keyword, /**< [in] single option keyword */
    RDM_TCHAR_T *optValue,      /**< [out] option value in string */
    size_t bytesIn,             /**< [in] # of bytes for \a optValue */
    size_t *bytesOut            /**< [out] # of bytes needed/used for output */
);

/** \brief Get RDM database specific options

    The function returns a semicolon-delimited list of keyword/value pairs for 
    all available options into the \a optSting buffer.

    \note If the \a optString argument is NULL, the function returns the
    required size in bytes of the destination string in \a bytesOut.

    \includedoc dox/sqldb-options.mdinc


    \retval{sOKAY}
    \retval{eINVOPTION}

    \see rdm_tfsAllocDatabase
    \see rdm_dbOpen
    \see rdm_dbGetOption
    \see rdm_dbSetOptions
    \see rdm_dbFree

    \ingroup db_initialization
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbGetOptions (
    RDM_DB db,              /**< [in] A valid RDM database handle */
    RDM_TCHAR_T *optString, /**< [out] option string */
    size_t bytesIn,         /**< [in] # of bytes for optString */
    size_t *bytesOut        /**< [out] result # of bytes */
);

/** \brief Get RDM database information

    The function returns a semicolon-delimited list of information keywords
    and returns information associated with the keyword.

    \a key should be a list of info values. If \a key is NULL, the function
    returns the pairs for all available options. If \a key is an empty string,
    the function returns an empty string.

    \retval{sOKAY}
    \retval{eINVOPTION}

    \see rdm_tfsAllocDatabase
    \see rdm_dbOpen
    \see rdm_dbFree

    \ingroup db_information
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbGetInfo (
    RDM_DB db,                  /**< [in] A valid RDM database handle */
    const RDM_TCHAR_T *keyword, /**< [in] An information keyword */
    RDM_TCHAR_T *infoString,    /**< [out] information string */
    size_t bytesIn,             /**< [in] # of bytes for optString */
    size_t *bytesOut            /**< [out] result # of bytes */
);

/** \brief Associate an ::RDM_CURSOR with rows based on a table id

    This function associates the ::RDM_CURSOR pointed to by \a pCursor with a
   set of rows in rowid order. The cursor will contain all of the rows for the
   table specified by \a tableId. The cursor will be initially positioned in the
    \e BeforeFirst position.

    \association_rules_db

    \par Locking Requirements
    None

    \retval{sOKAY}
    \retval{eCURSORDB}
    \retval{eDBNOTOPEN}
    \retval{eINVCURSORID}
    \retval{ePRECOMMITTED}

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbGetRowsAtRowId
    \see rdm_dbGetRowsByKey
    \see rdm_cursorGetRowsAtPosition
    \see rdm_cursorGetMemberRows
    \see rdm_cursorGetOwnerRow
    \see rdm_cursorGetRowsInReverseOrder
    \see rdm_cursorGetSelf
    \see rdm_cursorGetSiblingRows
    \see rdm_cursorGetClone
    \see rdm_cursorMoveToNext
    \see rdm_cursorReadRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_cursorLinkRow

    \ingroup db_association_rowid
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbGetRows (
    RDM_DB db,            /**< [in] A valid RDM database handle */
    RDM_TABLE_ID tableId, /**< [in] The table id for the row set */
    RDM_CURSOR *pCursor   /**< [out] A pointer to an ::RDM_CURSOR (must be
                             allocated, associated with \a db, or set to NULL) */
);

/** \brief Associate an ::RDM_CURSOR with rows based on a \a tableId and and \a rowId

    This function associates the ::RDM_CURSOR pointed to by \a pCursor with a row set
    ordered by \a rowid. The row set contains all of the rows for the specified
    table and will initially be positioned to the row specified by the rowid
    value.

    If \a rowid is not a valid rowid for the specified table an error will be
    returned and no rows will be associated with the cursor.

    If \a rowid is a valid rowid for the specified table but there is no row at
    that exact position then the function will return ::sNOTFOUND and the cursor
    will be placed in the ::CURSOR_BETWEEN position. Navigating forward
    (rdm_cursorMoveToNext()) from that position will position the cursor at
    the next rowid larger than the specified rowid. Navigating backward
    (rdm_cursorMoveToPrevious()) from that position will position the cursor
    at the next rowid smaller than the specified \a rowid.

    \association_rules_db

    \par Locking Requirements
    Read lock on the table \a tableId.

    \retval{sOKAY}
    \retval{sNOTFOUND} Success, but no row with the specified \a rowid, the cursor
   position is set to ::CURSOR_BETWEEN. \retval{eDBNOTOPEN} \retval{sNOTFOUND}
    \retval{eNOSTARTREAD}
    \retval{eNOTLOCKED}
    \retval{eCURSORDB}
    \retval{eINVROWIDVAL}
    \retval{ePRECOMMITTED}

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_dbGetRowsByKey
    \see rdm_cursorGetRowsAtPosition
    \see rdm_cursorGetMemberRows
    \see rdm_cursorGetOwnerRow
    \see rdm_cursorGetRowsInReverseOrder
    \see rdm_cursorGetSelf
    \see rdm_cursorGetSiblingRows
    \see rdm_cursorGetClone
    \see rdm_cursorMoveToNext
    \see rdm_cursorReadRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_cursorLinkRow

    \ingroup db_association_rowid
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbGetRowsAtRowId (
    RDM_DB db,            /**< [in] A valid RDM database handle */
    RDM_TABLE_ID tableId, /**< [in] The table identifier for the set of rows */
    RDM_ROWID_T rowId,    /**< [in] The rowid to position the cursor at */
    RDM_CURSOR *pCursor   /**< [out] A pointer to an ::RDM_CURSOR (must be
                             allocated, associated with \a db, or set to NULL) */
);

/** \brief Associate an ::RDM_CURSOR with a row set based on a key

    This function associates the ::RDM_CURSOR pointed to by \a pCursor with a
   row set ordered by the specified key. The row set will contain all of the
   rows for the table of the key specified. The ::RDM_CURSOR will be initially
   positioned in the \e BeforeFirst position.

    \association_rules_db

    \par Locking Requirements
    None.

    \retval{sOKAY}
    \retval{eCURSORDB}
    \retval{eDBNOTOPEN}
    \retval{eINVCURSORID}
    \retval{eINVKEYID}
    \retval{eISRTREE}
    \retval{ePRECOMMITTED}

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_dbGetRowsAtRowId
    \see rdm_dbGetRowsByKey
    \see rdm_dbGetRowsByKeyAtKey
    \see rdm_dbGetRowsByKeyAtSearchKey
    \see rdm_dbGetRowsByKeyInKeyRange
    \see rdm_cursorGetRowsByKeyAtPosition
    \see rdm_cursorGetMemberRows
    \see rdm_cursorGetOwnerRow
    \see rdm_cursorGetRowsInReverseOrder
    \see rdm_cursorGetSelf
    \see rdm_cursorGetSiblingRows
    \see rdm_cursorGetClone
    \see rdm_cursorMoveToNext
    \see rdm_cursorReadRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_cursorLinkRow

    \ingroup db_association_key
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbGetRowsByKey (
    RDM_DB db,          /**< [in] A valid RDM database handle */
    RDM_KEY_ID keyId,   /**< [in] The RDM key id */
    RDM_CURSOR *pCursor /**< [out] A pointer to an ::RDM_CURSOR (must be
                           allocated, associated with \a db, or set to NULL) */
);

/** \brief Associate an ::RDM_CURSOR with a row set that is ordered by key value
           and is initially positioned at the specified key value.

    This function associates a row set ordered by the specified key with the
    ::RDM_CURSOR pointed to by \a pCursor. The row set contains all of the rows
   for the table that the specified key is in.

    The ::RDM_CURSOR will be initially positioned at the first row that matches
    the specified key value. If there is not an exact match then the
   ::RDM_CURSOR will be placed in the ::CURSOR_BETWEEN position. Navigating
   forward (rdm_cursorMoveToNext()) from that position will position the
   ::RDM_CURSOR at the next key value larger than the specified key value.
   Navigating backward (rdm_cursorMoveToPrevious()) from that position will
   position the ::RDM_CURSOR at the next key value smaller than the specified
   key value.

    The engine validates the size of the buffers prior to accessing them.
    - If the size of the \a keyValue buffer is smaller than the size required to
    fully define the key then the remainder of the full buffer will be padded
    with NULL bytes and the search will be performed using that value.
    - If the size of the buffer is larger than the size required to fully
    the key then the extra bytes will be ignored and the search will be
    performed using the truncated value.

    \association_rules_db

    \par Locking Requirements
    Read lock on the table associated with \a keyId.

    \retval{sOKAY}
    \retval{sNOTFOUND} Success, but no exact key value match, the cursor
   position is set to ::CURSOR_BETWEEN. \retval{eNOSTARTREAD}
    \retval{eNOTLOCKED}
    \retval{eCURSORDB}
    \retval{eDBNOTOPEN}
    \retval{eINVCURSORID}
    \retval{eINVKEYID}
    \retval{eISRTREE}
    \retval{ePRECOMMITTED}

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_dbGetRowsAtRowId
    \see rdm_dbGetRowsByKey
    \see rdm_dbGetRowsByKeyAtKey
    \see rdm_dbGetRowsByKeyAtSearchKey
    \see rdm_dbGetRowsByKeyInKeyRange
    \see rdm_cursorGetMemberRows
    \see rdm_cursorGetOwnerRow
    \see rdm_cursorGetRowsInReverseOrder
    \see rdm_cursorGetSelf
    \see rdm_cursorGetSiblingRows
    \see rdm_cursorGetClone
    \see rdm_cursorMoveToNext
    \see rdm_cursorReadRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_cursorLinkRow
    \see \rdmcompile

    \ingroup db_association_key
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbGetRowsByKeyAtKey (
    RDM_DB db,            /**< [in] A valid RDM database handle */
    RDM_KEY_ID keyId,     /**< [in] The RDM key id  */
    const void *keyValue, /**< [in] The key value used to position the cursor */
    size_t len,           /**< [in] The length of the key value buffer */
    RDM_CURSOR *pCursor   /**< [out] A pointer to an ::RDM_CURSOR (must be
                             allocated, associated with \a db, or set to NULL) */
);

/** \brief Associate an ::RDM_CURSOR with a row set that is ordered by key value
    and is initially positioned at the specified key value.

    This function associates a row set ordered by the specified key with the
    ::RDM_CURSOR pointed to by \a pCursor. The row set contains all of the rows
   for the table that the specified key is in.

    The ::RDM_CURSOR will be initially positioned at the first row that matches
    the specified key value. If there is not an exact match then the
   ::RDM_CURSOR will be placed in the ::CURSOR_BETWEEN position. Navigating
   forward (rdm_cursorMoveToNext()) from that position will position the
   ::RDM_CURSOR at the next key value larger than the specified key value.
   Navigating backward (rdm_cursorMoveToPrevious()) from that position will
   position the ::RDM_CURSOR at the next key value smaller than the specified
   key value.

    The engine validates the size of the buffers prior to accessing them.
    - If the size of the \a keyValue buffer is smaller than the size required to
    fully define the key then the remainder of the full buffer will be padded
    with NULL bytes and the search will be performed using that value.
    - If the size of the buffer is larger than the size required to fully
    the key then the extra bytes will be ignored and the search will be
    performed using the truncated value.

    \association_rules_db

    \par Locking Requirements
    Read lock on the table associated with \a keyId.

    \retval{sOKAY}
    \retval{sNOTFOUND} Success, but no exact key value match, the cursor
   position is set to ::CURSOR_BETWEEN. \retval{eNOSTARTREAD}
    \retval{eNOTLOCKED}
    \retval{eCURSORDB}
    \retval{eDBNOTOPEN}
    \retval{eINVCURSORID}
    \retval{eINVKEYID}
    \retval{eISRTREE}
    \retval{ePRECOMMITTED}

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_dbGetRowsAtRowId
    \see rdm_dbGetRowsByKeyAtSearchKey
    \see rdm_dbGetRowsByKeyInKeyRange
    \see rdm_dbGetRowsByKeyInRangeKeyRange
    \see rdm_dbGetRowsByKeyInRtreeKeyRange
    \see rdm_dbGetRowsByKeyInSearchKeyRange
    \see rdm_cursorGetMemberRows
    \see rdm_cursorGetOwnerRow
    \see rdm_cursorGetRowsInReverseOrder
    \see rdm_cursorGetSelf
    \see rdm_cursorGetSiblingRows
    \see rdm_cursorGetClone
    \see rdm_cursorMoveToNext
    \see rdm_cursorReadRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_cursorLinkRow

    \ingroup db_association_key
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbGetRowsByKeyAtSearchKey (
    RDM_DB db,                      /**< [in] A valid RDM database handle */
    RDM_KEY_ID keyId,               /**< [in] The RDM key id  */
    const RDM_SEARCH_KEY *keyValue, /**< [in] The key value used to position the cursor */
    RDM_CURSOR *pCursor /**< [out] A pointer to an ::RDM_CURSOR (must be
                           allocated, associated with \a db, or set to NULL) */
);

/** \brief Associate an ::RDM_CURSOR with rows in a specified key range

    This function associates the ::RDM_CURSOR pointed to by \a pCursor with a
   row set ordered by the specified key. The row set contains all of the rows
   for the table that have a key value that is in the range specified by the
   start and end values. If the specified start value is NULL then the start of
   the range will be the first index value. If the end value is NULL then the
   end of the range will be the last index value. Specifying a start value of
   NULL and an end value of NULL is equivalent to rdm_dbGetRowsByKey().

    For a key that allows duplicate values it can be useful to specify the same
    value for the start and the end range. For example, the foreign side of a
    primary foreign key relationship can be retrieved by specifying the primary
    key value as the start and end value of a key on the foreign table that
    references the primary key of the primary table. If the index is unique
    then specifying the same start and end value will result in at most one row
    in the cursor collection.

    The cursor will be initially positioned to the \e BeforeFirst location.

    The engine checks the size of buffers prior to accessing them.
    - If the value of \a bytesInStartValue or \a bytesInEnd is zero, the size of
      \a startValue or \a endValue is assumed to be large enough to fully define
      the key.
    - If the value of \a bytesInStartValue or \a bytesInEnd is non-zero, the
      value must be large enough to fully define the key. Otherwise, an error
      is returned.
    - If the size of the buffer is larger than the size required to fully
      the key then the extra bytes will be ignored and the search will be
      performed using the truncated value.

    \association_rules_db

    \par Locking Requirements
    None.

    \retval{sOKAY}
    \retval{sNOTFOUND} Success, but no exact key value match, the cursor
   position is set to ::CURSOR_BETWEEN. \retval{eCURSORDB} \retval{eDBNOTOPEN}
    \retval{eINVCURSORID}
    \retval{eINVKEYID}
    \retval{eISRTREE}
    \retval{ePRECOMMITTED}

    \ingroup db_association_key

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_dbGetRowsAtRowId
    \see rdm_dbGetRowsByKey
    \see rdm_dbGetRowsByKeyAtSearchKey
    \see rdm_dbGetRowsByKeyInRangeKeyRange
    \see rdm_dbGetRowsByKeyInRtreeKeyRange
    \see rdm_dbGetRowsByKeyInSearchKeyRange
    \see rdm_cursorGetMemberRows
    \see rdm_cursorGetOwnerRow
    \see rdm_cursorGetRowsInReverseOrder
    \see rdm_cursorGetSelf
    \see rdm_cursorGetSiblingRows
    \see rdm_cursorGetClone
    \see rdm_cursorMoveToNext
    \see rdm_cursorReadRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_cursorLinkRow
    \see \rdmcompile
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbGetRowsByKeyInKeyRange (
    RDM_DB db,                /**< [in] A valid RDM database handle */
    RDM_KEY_ID keyId,         /**< [in] The RDM key id */
    const void *startValue,   /**< [in] The value for the start of the range */
    size_t bytesInStartValue, /**< [in] the size of startValue in bytes */
    const void *endValue,     /**< [in] The value for the end of the range */
    size_t bytesInEndValue,   /**< [in] the size of endValue in bytes */
    RDM_CURSOR *pCursor       /**< [out] A pointer to an ::RDM_CURSOR (must be
                                 allocated, associated with \a db, or set to NULL) */
);

/** \brief Associate an ::RDM_CURSOR with rows in a specified key range

    This function associates the ::RDM_CURSOR pointed to by \a pCursor with a
   row set ordered by the specified key. The row set contains all of the rows
   for the table that have a key value that is in the range specified by the
   start and end values. If the specified start value is NULL then the start of
   the range will be the first index value stored in the database. If the end
   value is NULL then the end of the range will be the last index value stored
   in the database. Specifying a start value of NULL and an end value of NULL is
   equivalent to rdm_dbGetRowsByKey().

    For a key that allows duplicate values it can be useful to specify the same
    value for the start and the end range. For example, the foreign side of a
    primary foreign key relationship can be retrieved by specifying the primary
    key value as the start and end value of a key on the foreign table that
    references the primary key of the primary table. If the index is unique
    then specifying the same start and end value will result in at most one row
    in the cursor collection.

    The range can either be 'inclusive' or 'exclusive'. If the range field in
    the ::RDM_RANGE_KEY is set to ::RDM_RANGE_OPEN keys that match the value
   will be included in the result set. If the range field in ::RDM_RANGE_KEY is
    ::RDM_RANGE_CLOSED then keys that match the value will not be included in
   the result set.

    The cursor will be initially positioned to the \e BeforeFirst location.

    \association_rules_db

    \par Locking Requirements
    None.

    \retval{sOKAY}
    \retval{sNOTFOUND} Success, but no exact key value match, the cursor
   position is set to ::CURSOR_BETWEEN. \retval{eCURSORDB} \retval{eDBNOTOPEN}
    \retval{eINVCURSORID}
    \retval{eINVKEYID}
    \retval{eISRTREE}
    \retval{ePRECOMMITTED}

    \ingroup db_association_key

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_dbGetRowsAtRowId
    \see rdm_dbGetRowsByKey
    \see rdm_dbGetRowsByKeyInKeyRange
    \see rdm_dbGetRowsByKeyInRtreeKeyRange
    \see rdm_dbGetRowsByKeyInSearchKeyRange
    \see rdm_cursorGetMemberRows
    \see rdm_cursorGetOwnerRow
    \see rdm_cursorGetRowsInReverseOrder
    \see rdm_cursorGetSelf
    \see rdm_cursorGetSiblingRows
    \see rdm_cursorGetClone
    \see rdm_cursorMoveToNext
    \see rdm_cursorReadRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_cursorLinkRow
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbGetRowsByKeyInRangeKeyRange (
    RDM_DB db,                       /**< [in] A valid RDM database handle */
    RDM_KEY_ID keyId,                /**< [in] The RDM key id */
    const RDM_RANGE_KEY *startValue, /**< [in] The value for the start of the range */
    const RDM_RANGE_KEY *endValue, /**< [in] The value for the end of the range */
    RDM_CURSOR *pCursor /**< [out] A pointer to an ::RDM_CURSOR (must be
                           allocated, associated with \a db, or set to NULL) */
);

/** \brief Associate an ::RDM_CURSOR with a row set based on a r-tree

    This function associates the ::RDM_CURSOR pointed to by \a pCursor with a
   row set based on the the specified rtree index. The row set will contain all
   of the rows for the table of the key specified that match the search
   criteria. The
    ::RDM_CURSOR will be initially positioned in the \e BeforeFirst position.

    \association_rules_db

    \par Locking Requirements
    None.

    \retval{sOKAY}
    \retval{sNOTFOUND} Success, but no exact key value match, the cursor
   position is set to ::CURSOR_BETWEEN. \retval{eCURSORDB} \retval{eDBNOTOPEN}
    \retval{eINVCURSORID}
    \retval{eINVKEYID}
    \retval{eNOTRTREE}
    \retval{ePRECOMMITTED}

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_dbGetRowsAtRowId
    \see rdm_dbGetRowsByKey
    \see rdm_dbGetRowsByKeyInKeyRange
    \see rdm_dbGetRowsByKeyInRangeKeyRange
    \see rdm_dbGetRowsByKeyInSearchKeyRange
    \see rdm_cursorGetMemberRows
    \see rdm_cursorGetOwnerRow
    \see rdm_cursorGetRowsInReverseOrder
    \see rdm_cursorGetSelf
    \see rdm_cursorGetSiblingRows
    \see rdm_cursorGetClone
    \see rdm_cursorMoveToNext
    \see rdm_cursorReadRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_cursorLinkRow

    \ingroup db_association_key
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbGetRowsByKeyInRtreeKeyRange (
    RDM_DB db,                     /**< [in] A valid RDM database handle */
    RDM_KEY_ID keyId,              /**< [in] The RDM key id */
    const RDM_RTREE_KEY *keyValue, /**< [in] The key value used to position the cursor */
    RDM_CURSOR *pCursor /**< [out] A pointer to an ::RDM_CURSOR (must be
                           allocated, associated with \a db, or set to NULL) */
);

/** \brief Associate an ::RDM_CURSOR with rows in a specified key range

    This function associates the ::RDM_CURSOR pointed to by \a pCursor with a
   row set ordered by the specified key. The row set contains all of the rows
   for the table that have a key value that is in the range specified by the
   start and end values. If the specified start value is NULL then the start of
   the range will be the first index value. If the end value is NULL then the
   end of the range will be the last index value. Specifying a start value of
   NULL and an end value of NULL is equivalent to rdm_dbGetRowsByKey().

    For a key that allows duplicate values it can be useful to specify the same
    value for the start and the end range. For example, the foreign side of a
    primary foreign key relationship can be retrieved by specifying the primary
    key value as the start and end value of a key on the foreign table that
    references the primary key of the primary table. If the index is unique
    then specifying the same start and end value will result in at most one row
    in the cursor collection.

    The cursor will be initially positioned to the \e BeforeFirst location.

    The engine validates the size of buffers prior to accessing them.
    - If the size of the \a startValue or \a endValue buffer is smaller than the
   size required to fully define the key then the remainder of the full buffer
   will be padded with NULL bytes and the search will be performed using that
   value.
    - If the size of the buffer is larger than the size required to fully
    the key then the extra bytes will be ignored and the search will be
    performed using the truncated value.

    \association_rules_db

    \par Locking Requirements
    None.

    \retval{sOKAY}
    \retval{sNOTFOUND} Success, but no exact key value match, the cursor
   position is set to ::CURSOR_BETWEEN. \retval{eCURSORDB} \retval{eDBNOTOPEN}
    \retval{eINVCURSORID}
    \retval{eINVKEYID}
    \retval{eISRTREE}
    \retval{ePRECOMMITTED}

    \ingroup db_association_key

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_dbGetRowsAtRowId
    \see rdm_dbGetRowsByKeyAtSearchKey
    \see rdm_dbGetRowsByKeyInKeyRange
    \see rdm_dbGetRowsByKeyInRangeKeyRange
    \see rdm_dbGetRowsByKeyInRtreeKeyRange
    \see rdm_cursorGetMemberRows
    \see rdm_cursorGetOwnerRow
    \see rdm_cursorGetRowsInReverseOrder
    \see rdm_cursorGetSelf
    \see rdm_cursorGetSiblingRows
    \see rdm_cursorGetClone
    \see rdm_cursorMoveToNext
    \see rdm_cursorReadRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_cursorLinkRow
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbGetRowsByKeyInSearchKeyRange (
    RDM_DB db,                        /**< [in] A valid RDM database handle */
    RDM_KEY_ID keyId,                 /**< [in] The RDM key id */
    const RDM_SEARCH_KEY *startValue, /**< [in] The value for the start of the range */
    const RDM_SEARCH_KEY *endValue, /**< [in] The value for the end of the range */
    RDM_CURSOR *pCursor /**< [out] A pointer to an ::RDM_CURSOR (must be
                           allocated, associated with \a db, or set to NULL) */
);

/** \brief Get the RDM_TFS handle associated with a \a db

    This function assigns the ::RDM_TFS handle associated with an ::RDM_DB to
    \a pTFS.

    \par Locking Requirements
    None.

    \retval{sOKAY}

    \see rdm_tfsAllocDatabase
    \see rdm_dbOpen
    \see rdm_dbGetTFSType

    \ingroup db_information
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbGetTFS (
    RDM_DB db, /**< [in] A valid RDM database handle */
    RDM_TFS *pTFS /**< [out] A pointer to an ::RDM_TFS that will be assigned the TFS handle associated with \a db */);

/** \brief Get the type of the ::RDM_TFS handle associated with a database

    This function assigns the type of the ::RDM_TFS handle associated with
    an ::RDM_DB to \a pTfsType

    \par Locking Requirements
    None.

    \retval{sOKAY}

    \see rdm_tfsAllocDatabase
    \see rdm_dbOpen
    \see rdm_dbGetTFS

    \ingroup db_information
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbGetTFSType (
    RDM_DB db,         /**< [in] A valid RDM database handle */
    TFS_TYPE *pTfsType /**< [out] Type of TFS */
);

/** \brief Get the transaction status for a database handle

    This function returns the current transaction status of the database
    handle.

    ::RDM_TRANS_STATUS values that can be returned in \a status are:

    | Status                 | Description                       |
    |:-----------------------|:----------------------------------|
    | ::RDM_TRANS_READ       | \copydetails RDM_TRANS_READ       |
    | ::RDM_TRANS_UPDATE     | \copydetails RDM_TRANS_UPDATE     |
    | ::RDM_TRANS_INITIALIZE | \copydetails RDM_TRANS_INITIALIZE |
    | ::RDM_TRANS_SNAPSHOT   | \copydetails RDM_TRANS_SNAPSHOT   |
    | ::RDM_TRANS_NONE       | \copydetails RDM_TRANS_NONE       |
    | ::RDM_TRANS_UNKNOWN    | \copydetails RDM_TRANS_UNKNOWN    |

    \par Locking Requirements
    None

    \retval{sOKAY}
    \retval{eDBNOTOPEN}

    \see rdm_dbOpen
    \see rdm_dbGetLockStatus
    \see rdm_cursorGetLockStatus
    \see rdm_dbGetTransactionStatus
    \see rdm_dbStartUpdate
    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbGetRows
    \see rdm_dbEnd
    \see rdm_transEnd

    \ingroup db_locking
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbGetTransactionStatus (
    RDM_DB db,               /**< [in] A valid RDM database handle */
    RDM_TRANS_STATUS *status /**< [out] The current transaction status for the
                               database handle */
);

/** \brief Database import API

    This API imports data from text files to database tables. The import
    functionality is simple enough to allow great quantities of raw ASCII/XML
    data to be entered into an RDM database with very little specification.
    With more complex requirements, rdm_dbImport() can be instructed to
    perform mappings between the input data and table rows/columns.

    For CSV imports all instructions to rdm_dbImport() are placed into a text
    file in the form of an import specification language (ISL). An ISL
    specification identifies the database and the files containing ASCII data to
    be imported. It also contains mappings between ASCII data fields and the
    RDM tables.

    For XML imports the XML document must be in the format supported be the RDM
    engine. The RDM rdm_dbExport() API will create an XML document that is
    in the correct format for rdm_dbImport(). See the documentation on the
    XML format for full details.

    This function is located in \lib{export}

    \par Locking Requirements
    Write locks on the tables being imported.

    \retval{sOKAY}
    \retval{eDBNOTOPEN}

    \ingroup db_utilities

    \see rdm_dbStartUpdate
    \see rdm_dbExport
    \see rdm_dbEnd
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbImport (
    RDM_DB db,                   /**< [in] A valid RDM database handle */
    const RDM_TCHAR_T *optString /**< [in] A semicolon delimited string that
                                    contains the utility options */
);

/** \brief Insert a new row into a table at the specified rowId

    This function adds a new row to the table specified by \a tableId. It can
    optionally associate the ::RDM_CURSOR pointed to by \a pCursor with a singleton
    containing the inserted row.

    The row will be created in the table specified by \a tableId.

    The engine validates the size of the buffers prior to accessing them.
    - If the size of the \a colValues buffer is smaller than the size required to
    fully define a row then the ::eBADDATALEN error code will be returned
    and no row will be added to the table.
    - If the size of the buffer is larger than the size required to fully
    define a row then the extra bytes will be ignored and no status will be
    returned.

    For each column that is not designated as NOT NULL a \c{_has_value} field is
    added to the structure. If the \c{_has_value} field for a column is not set
    (::RDM_COL_IS_NULL), then the value for the column will be considered NULL
    and anything in the structure value field for the column will be ignored.
    If the column has data, the \c{_has_value} field should be set to ::RDM_COL_HAS_VALUE.

    \association_rules_db

    \par Locking Requirements
    Write lock on the table specified by \a tableId.  Foreign keys also require
    read locks on the referenced tables.

    \retval{sOKAY}
    \retval{eNOSTARTUPDATE}
    \retval{eNOTLOCKED}
    \retval{eCURSORDB}
    \retval{eDBNOTOPEN}
    \retval{ePRECOMMITTED}
    \retval{eDUPLICATE}
    \retval{eREADONLY}
    \retval{eREFINTEGRITY}
    \retval{eROWLIMIT}

    \see rdm_dbStartUpdate
    \see rdm_dbInsertRows
    \see rdm_dbReinsertRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_dbAlterCatalog
    \see rdm_dbEnd
    \see \rdmcompile

    \ingroup db_insert
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbInsertRow (
    RDM_DB db,            /**< [in] A valid RDM database handle */
    RDM_TABLE_ID tableId, /**< [in] The table identifier for table to insert the
                            row into */
    const void *colValues, /**< [in] A buffer that contains the column values of
                             the row to be inserted formatted as the table
                             structure generated when the database schema was
                             compiled. If the table definition has changed
                             since the schema was first compiled a new header
                             file will need to be generated. */
    size_t bytesIn,        /**< [in] The length of the \a colValues buffer.
                             If \a colValues is NULL this value should be 0. */
    RDM_CURSOR *pCursor /**< [out] A optional pointer to an ::RDM_CURSOR (must
                           be allocated, associated with \a db, or set to NULL) */
);

/** \brief Insert rows into a table

    This function inserts rows to the table specified by \a tableId.

    If \a pCursor is not NULL the ::RDM_CURSOR pointed to by \a pCursor will
    be associated with the collection of inserted rows. In the case
    that there are errors inserting there may be fewer rows in the resulting
    ::RDM_CURSOR than \a num.

    If \a pCursorStatus is not NULL the ::RDM_CURSOR pointed to by \a
   pCursorStatus will be associated with a set of ::RDM_RETCODE values
   pertaining to the status of each row that was to be inserted into the table.

    For each column that is not designated as NOT NULL a \c{_has_value} field is
    added to the table structure. If the \c{_has_value} field for a column is
   not set the value for the column will be considered NULL and anything in the
    structure value field for the column will be ignored.

    \association_rules_db

    \par Locking Requirements
    Write lock on the table specified by \a tableId. Foreign keys also require
    read locks on the referenced tables.

    \retval{sOKAY}
    \retval{eNOSTARTUPDATE}
    \retval{eNOTLOCKED}
    \retval{eCURSORDB}
    \retval{eDBNOTOPEN}
    \retval{ePRECOMMITTED}
    \retval{eDUPLICATE}
    \retval{eREFINTEGRITY}
    \retval{eREADONLY}
    \retval{eROWLIMIT}
    \retval{sOKAYWITHINFO} One or more of the inserted rows failed to be
   inserted.  Iterate \a pCursorStatus to retreve the individual statuses by
   calling ::rdm_cursorGetRowStatus

    \see rdm_dbStartUpdate
    \see rdm_dbInsertRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_dbAlterCatalog
    \see rdm_dbEnd
    \see \rdmcompile

    \ingroup db_insert
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbInsertRows (
    RDM_DB db,            /**< [in] A valid RDM database handle */
    RDM_TABLE_ID tableId, /**< [in] The table identifier for table to insert the
                            row into */
    const void *colValues, /**< [in] An array that contains the column values of
                             the rows to be inserted formatted as an array of
                             the table structure generated when the database
                             schema was compiled. If the table definition has
                             changed since the schema was first compiled a new
                             header file will need to be generated. */
    size_t bytesIn, /**< [in] The size of the \a colValues buffer (in bytes) */
    uint32_t num,   /**< [in] The number of array entries (the number of
                      row to be added to the table */
    RDM_CURSOR *pCursorStatus, /**< [out] A pointer to an ::RDM_CURSOR (must be
                                  allocated, associated with \a db, or set to
                                  NULL). There will be one RDM_RETCODE value for
                                  each row that was attempted to be added to the
                                  table. This parameter can be NULL */
    RDM_CURSOR *pCursor        /**< [out] A pointer to an ::RDM_CURSOR (must be
                                  allocated, associated with \a db, or set to NULL).
                                  This will contain a collection of all the rows that
                                  were inserted. This parameter can be NULL. */
);

/** \brief Open an existing RDM database using the specified database handle

    This function opens a database for access in a shared, read-only, or
    exclusive mode.

    If the specified handle already has an open database associate with it that
    database will be closed.

    \par Database Unions

    A database union is a unified view of the data in more than one identically
    structured database. It makes the multiple databases appear as one. A union
    of multiple databases differs from having multiple databases open in that:
    - unioned databases must have identical structures
    - the union is viewed as one database, using one database handle
    - it is read-only

    Database unions are intended to be used with distributed databases or
    database mirrors to create a single, merged view of data that is owned
    and updated in separate locations by separate entities.

    A complete database specification can be a union of 2 or more databases,
    which is represented by database names separated by "|" symbols:
    - db_namespec[ | db_namespec]...

    Each database included in a union must have an identical table definition.
    When opened as a union, the data will appear to come from one database,
    whether it is read through record or key scanning.

    A union is read only, and an error will be returned if a union is specified
    but the open mode is not ::RDM_OPEN_READONLY.

    \par Locking Requirements
    None.

    \retval{sOKAY}
    \retval{eINVDBNAMESPEC}
    \retval{eINVENCRYPT}
    \retval{eLOCALEMISMATCH}
    \retval{eNODB}
    \retval{eUNAVAIL}
    \retval{eOPENMODE}
    \retval{sTRIGGERSREADONLY}

    \see rdm_rdmAllocDatabase
    \see rdm_dbSetOptions
    \see rdm_dbSetEncrypt
    \see rdm_dbEncrypt
    \see rdm_dbTriggersOn
    \see rdm_dbSetCatalog
    \see rdm_dbLoadCatalog
    \see rdm_dbCompileCatalog
    \see rdm_dbGetRows
    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbClose
    \see rdm_dbFree

    \ingroup db_initialization
    */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbOpen (
    RDM_DB db,                     /**< [in] An allocated ::RDM_DB */
    const RDM_TCHAR_T *dbNameSpec, /**< [in] The database name specification */
    RDM_OPEN_MODE mode             /**< [in] The database open mode */
);

/** \brief Persist in-memory tables to disk

  This function will write data from in-memory tables to the physical file(s)
    backing the database. The in-memory tables must be
  \"\c{storage=inmemory_persist}\" or \"\c{storage=inmemory_read}\".

    \par Locking Requirements
    Read locks on all tables (::RDM_LOCK_ALL).

    \retval{sOKAY}
    \retval{eDBNOTOPEN}
    \retval{eREADONLY}
    \retval{eNOSTARTREAD}
    \retval{eNOTLOCKED}
    \retval{ePRECOMMITTED}

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbExport
    \see rdm_dbEnd

    \ingroup db_miscellaneous
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbPersistInMemory (
    RDM_DB db /**< [in] A valid RDM database handle */
);

/** \brief Precommit a transactional operation (Two-Phase Commit phase 1)

    rdm_dbPrecommit() performs phase one of a two-phase commit. The function
   prepares to commit a transaction by writing to the database log file all
   changes made since the latest root transaction for the database was started.
   Soon after this call, rdm_dbEnd(), rdm_transEnd(), rdm_dbEndRollback() or
   rdm_transEndRollback() should be called to finally commit or rollback the
   transaction. The difference between calling rdm_dbPrecommit() prior to
   rdm_dbEnd() is that all file I/O necessary to prepare the transaction will be
   done during the precommit, and all rdm_dbEnd() / rdm_transEnd() has left to
   do is a minimal task, essentially flipping a switch to commit the prepared
    transaction.

    This call does not change the status of locks, so a database should not be
    held in this state indefinitely.

    \par Locking Requirements
    None.

    \retval{sOKAY}
    \retval{eDBNOTOPEN}
    \retval{eNOSTARTUPDATE}
    \retval{ePRECOMMITTED}

    \ingroup db_locking
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbPrecommit (
    RDM_DB db /**< [in] A valid RDM database handle */
);

/** \brief Rebuild keys for an index

    This function allows an application to rebuild keys for the index specified
    by \a keyId.

    If \a reportFrequency is non-zero and \a fcn is not NULL, rdm_dbRebuildKey()
    calls the report function pointed to by \a fcn every \a reportFrequency rows
    it has successfully processed.

    \par Locking Requirements
    The table the key is on must be write locked.

    \retval{sOKAY}
    \retval{eDBNOTOPEN}
    \retval{eREADONLY}
    \retval{eNOSTARTUPDATE}
    \retval{eNOTLOCKED}
    \retval{ePRECOMMITTED}

    \ingroup db_utilities

    \see rdm_dbStartUpdate
    \see rdm_dbImport
    \see rdm_dbEnd
    \see rdm_dbRebuildKeys
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbRebuildKey (
    RDM_DB db,                /**< [in] A valid RDM database handle */
    RDM_KEY_ID keyId,         /**< [in] The RDM key identifier */
    uint32_t reportFrequency, /**< [in] how often to report status on the rebuild */
    RDM_REBUILD_INDEX_REPORT_FCN
        *fcn /**< [in] A function used to report status on the rebuild */
);

/** \brief Rebuild keys for one or more indexes

    This function allows an application to rebuild the indexes specified by
    \a keyId.

    If \a reportFrequency is non-zero and \a fcn is not NULL,
   rdm_dbRebuildKeys() calls the report function pointed to by \a fcn every \a
   reportFrequency rows it has successfully processed.

    \par Locking Requirements
    The table for each key specified must be write locked.

    \retval{sOKAY}
    \retval{eDBNOTOPEN}
    \retval{eREADONLY}
    \retval{eNOSTARTUPDATE}
    \retval{eNOTLOCKED}
    \retval{ePRECOMMITTED}

    \ingroup db_utilities

    \see rdm_dbStartUpdate
    \see rdm_dbImport
    \see rdm_dbEnd
    \see rdm_dbRebuildKey
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbRebuildKeys (
    RDM_DB db,               /**< [in] A valid RDM database handle */
    uint32_t numKeys,        /**< [in] The number of keys to rebuild*/
    const RDM_KEY_ID *keyId, /**< [in] The RDM key identifier */
    uint32_t reportFrequency, /**< [in] how often to report status on the rebuild */
    RDM_REBUILD_INDEX_REPORT_FCN
        *fcn /**< [in] A function used to report status on the rebuild */
);

/** \brief Insert a new row into a table

    This function adds a new row to the table specified by \a tableId at the
    specified \a rowId. It can optionally associate the ::RDM_CURSOR pointed to by
    \a pCursor with a singleton containing the inserted row.

    The row will be created in the table specified by \a tableId.

    The row will be placed in the specified \a rowId location. An error will be
    returned if that \a rowId already as a row associated with it or if the \a rowId
    is greater than the total number of rows currently in the table.

    The engine validates the size of the buffers prior to accessing them.
    - If the size of the \a colValues buffer is smaller than the size required to
    fully define a row then the ::eBADDATALEN error code will be returned
    and no row will be added to the table.
    - If the size of the buffer is larger than the size required to fully
    define a row then the extra bytes will be ignored and no status will be
    returned.

    For each column that is not designated as NOT NULL a \c{_has_value} field is
    added to the structure. If the \c{_has_value} field for a column is not set
    (::RDM_COL_IS_NULL), then the value for the column will be considered NULL
    and anything in the structure value field for the column will be ignored.
    If the column has data, the \c{_has_value} field should be set to ::RDM_COL_HAS_VALUE.

    \association_rules_db

    \par Locking Requirements
    Write lock on the table specified by \a tableId.

    \retval{sOKAY}
    \retval{eNOSTARTUPDATE}
    \retval{eNOTLOCKED}
    \retval{eCURSORDB}
    \retval{eDBNOTOPEN}
    \retval{ePRECOMMITTED}
    \retval{eDUPLICATE}
    \retval{eREADONLY}
    \retval{eREFINTEGRITY}
    \retval{eROWLIMIT}

    \see rdm_dbStartUpdate
    \see rdm_dbInsertRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_dbAlterCatalog
    \see rdm_dbEnd
    \see \rdmcompile

    \ingroup db_insert
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbReinsertRow (
    RDM_DB db, /**< [in] A valid RDM database handle */
    RDM_TABLE_ID tableId, /**< [in] The table identifier for table to insert the row into */
    RDM_ROWID_T rowId,     /**< [in] The rowid to be assigned to the row */
    const void *colValues, /**< [in] A buffer that contains the column values of
                             the row to be inserted formatted as the table
                             structure generated when the database schema was
                             compiled. If the table definition has changed
                             since the schema was first compiled a new header
                             file will need to be generated. */
    size_t bytesIn,        /**< [in] The length of the \a colValues buffer. If
                             \a colValues is NULL this value should be 0. */
    RDM_CURSOR *pCursor /**< [out] A optional pointer to an ::RDM_CURSOR (must be
                          allocated, associated with \a db, or set to NULL) */
);

/** \brief Associate a catalog with an allocated database

    This function associates a catalog with an allocated database. The ::RDM_DB
    must not have a database open. When the database is opened with
    rdm_dbOpen() the associated catalog will be used. If a catalog is found
    on disk then the specified catalog and stored catalog will be compared to
    ensure that they are compatible.

    Catalog can either be in the compiled format or can be a series of SQL
    statements.

    \par Locking Requirements
    None

    \retval{sOKAY}
    \retval{eDBOPEN}
    \retval{eBADCAT}
    \retval{eDDL_SYNTAX}
    \retval{ePRECOMMITTED}
    \retval{eLOCALEMISMATCH}

    \ingroup db_initialization

    \see rdm_tfsAllocDatabase
    \see rdm_dbSetCatalogFromFile
    \see rdm_dbLoadCatalogFromFile
    \see rdm_dbSetCatalogFromFile
    \see rdm_dbAlterCatalog
    \see rdm_dbOpen
    \see ddl_ug
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbSetCatalog (
    RDM_DB db, /**< [in] The allocated, but unopened ::RDM_DB the catalog will be associated with*/
    const RDM_TCHAR_T
        *catalog /**< [in] The catalog definition to be associated with \a db */
);

/** \brief Associate a catalog with an allocated database

    This function associates a catalog with an allocated database. The ::RDM_DB
    must not have a database open. When the database is opened with
    rdm_dbOpen() the associated catalog will be used. If a catalog is found
    on disk then the specified catalog and stored catalog will be compared to
    ensure that they are compatible.

    Catalog can either be a catalog compiled RDM catalog file (\c{.cat}) or it
   can be a file containing a series for SQL DDL commands (\c{.sdl}). An error
   will be returned if a file with any other extension is specified.

    \par Locking Requirements
    None.

    \retval{sOKAY}
    \retval{eDBOPEN}
    \retval{eBADCAT}
    \retval{eDDL_SYNTAX}
    \retval{ePRECOMMITTED}
    \retval{eLOCALEMISMATCH}
    \retval{eFILEOPEN}

    \ingroup db_initialization

    \see rdm_tfsAllocDatabase
    \see rdm_dbSetCatalogFromFile
    \see rdm_dbAlterCatalog
    \see rdm_dbOpen

    \see rdm_dbFree
    \see rdm_dbOpen
    \see rdm_dbSetCatalog
    \see rdm_dbAlterCatalog
    \see ddl_ug
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbSetCatalogFromFile (
    RDM_DB db, /**< [in] The allocated, but unopened ::RDM_DB the catalog will be associated with*/
    const RDM_TCHAR_T *catfile /**< [in] The name of the file containing the catalog */
);

/** \brief Associate a schema with an allocated database

    This function associates a catalog with an allocated database. The ::RDM_DB
    must not have a database open. When the specified schema is compiled the
    resulting catalog is associated with the database handle and will be used
    in a subsequent call to rdm_dbOpen(). If a catalog is found on disk then the
    resulting catalog and stored catalog will be compared to ensure that they
    are compatible.

    The format of the schema is a series of SQL statements.

    \par Locking Requirements
    None

    \retval{sOKAY}
    \retval{eDBOPEN}
    \retval{eBADCAT}
    \retval{eDDL_SYNTAX}
    \retval{ePRECOMMITTED}
    \retval{eLOCALEMISMATCH}

    \ingroup db_initialization

    \see rdm_tfsAllocDatabase
    \see rdm_dbSetCatalogFromFile
    \see rdm_dbLoadCatalogFromFile
    \see rdm_dbAlterCatalog
    \see rdm_dbOpen
    \see ddl_ug
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbCompileCatalog (
    RDM_DB db, /**< [in] The allocated, but unopened ::RDM_DB the catalog will be associated with*/
    const RDM_TCHAR_T
        *schema /**< [in] The schema definition to be associated with \a db */
);

/** \brief Associate a catalog with an allocated database

    This function associates a catalog with an allocated database. The ::RDM_DB
    must not have a database open. When the specified schema is compiled the
    resulting catalog is associated with the database handle and will be used
    in a subsequent call to rdm_dbOpen(). If a catalog is found on disk then the
    resulting catalog and stored catalog will be compared to ensure that they
    are compatible.


   Schemafile must be a file containing a series for SQL DDL commands (\c{.sdl}).
   An error will be returned if a file with any other extension is specified.

    \par Locking Requirements
    None.

    \retval{sOKAY}
    \retval{eDBOPEN}
    \retval{eBADCAT}
    \retval{eDDL_SYNTAX}
    \retval{ePRECOMMITTED}
    \retval{eLOCALEMISMATCH}
    \retval{eFILEOPEN}

    \ingroup db_initialization

    \see rdm_tfsAllocDatabase
    \see rdm_dbSetCatalogFromFile
    \see rdm_dbLoadCatalogFromFile
    \see rdm_dbAlterCatalog
    \see rdm_dbOpen

    \see rdm_dbFree
    \see rdm_dbOpen
    \see rdm_dbSetCatalog
    \see rdm_dbLoadCatalog
    \see rdm_dbAlterCatalog
    \see ddl_ug
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbCompileCatalogFromFile (
    RDM_DB db, /**< [in] The allocated, but unopened ::RDM_DB the catalog will be associated with*/
    const RDM_TCHAR_T *schemafile /**< [in] The name of the file containing the catalog */
);

/** \brief Associate a catalog with an allocated database

    This function associates a catalog with an allocated database. The ::RDM_DB
    must not have a database open. When the database is opened with
    rdm_dbOpen() the associated catalog will be used. If a catalog is found
    on disk then the specified catalog and stored catalog will be compared to
    ensure that they are compatible.

    Catalog must be in the compiled format.

    \par Locking Requirements
    None

    \retval{sOKAY}
    \retval{eDBOPEN}
    \retval{eBADCAT}
    \retval{eDDL_SYNTAX}
    \retval{ePRECOMMITTED}
    \retval{eLOCALEMISMATCH}

    \ingroup db_initialization

    \see rdm_tfsAllocDatabase
    \see rdm_dbLoadCatalogFromFile
    \see rdm_dbSetCatalogFromFile
    \see rdm_dbCompileCatalogFromFile
    \see rdm_dbSetCatalog
    \see rdm_dbCompileCatalog
    \see rdm_dbAlterCatalog
    \see rdm_dbOpen
    \see ddl_ug
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbLoadCatalog (
    RDM_DB db, /**< [in] The allocated, but unopened ::RDM_DB the catalog will be associated with*/
    const RDM_TCHAR_T
        *catalog /**< [in] The catalog definition to be associated with \a db */
);

/** \brief Associate a catalog with an allocated database

    This function associates a catalog with an allocated database. The ::RDM_DB
    must not have a database open. When the database is opened with
    rdm_dbOpen() the associated catalog will be used. If a catalog is found
    on disk then the specified catalog and stored catalog will be compared to
    ensure that they are compatible.

   catfile can must be a catalog compiled RDM catalog file (\c{.cat}). An error
   will be returned if a file with any other extension is specified.

    \par Locking Requirements
    None.

    \retval{sOKAY}
    \retval{eDBOPEN}
    \retval{eBADCAT}
    \retval{eDDL_SYNTAX}
    \retval{ePRECOMMITTED}
    \retval{eLOCALEMISMATCH}
    \retval{eFILEOPEN}

    \ingroup db_initialization

    \see rdm_tfsAllocDatabase
    \see rdm_dbSetCatalogFromFile
    \see rdm_dbCompileCatalogFromFile
    \see rdm_dbAlterCatalog
    \see rdm_dbOpen

    \see rdm_dbFree
    \see rdm_dbOpen
    \see rdm_dbLoadCatalog
    \see rdm_dbCompileCatalog
    \see rdm_dbSetCatalog
    \see rdm_dbAlterCatalog
    \see ddl_ug
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbLoadCatalogFromFile (
    RDM_DB db, /**< [in] The allocated, but unopened ::RDM_DB the catalog will be associated with*/
    const RDM_TCHAR_T *catfile /**< [in] The name of the file containing the catalog */
);

/** \brief Associate an encryption context with a database handle

    This function will associate an encryption context to a database handle.
    When a database is opened the system will us the encryption contexts
    associated with the handle to attempt to open the database.

    Typically you would associate the encryption context to a handle prior to
    open a database.

    You can remove all encryption contexts associated with the handle by passing
    in a NULL encryption context to this function.

    \par Locking Requirements
    None.

    \retval{sOKAY}
    \retval{eDBOPEN}
    \retval{ePRECOMMITTED}

    \see rdm_tfsAllocEncrypt
    \see rdm_dbEncrypt
    \see rdm_dbGetEncrypt

    \ingroup db_encryption
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbSetEncrypt (
    RDM_DB db,      /**< [in] A valid RDM database handle */
    RDM_ENCRYPT enc /**< [in] An RDM_ENCRYPT to associate with \a db (can be
                       NULL to remove \e all contexts associated with \a db) */
);

/** \brief Replace default error reporting with a custom function

    This function accepts a pointer to a function that will be called when RDM
    detects an error. The provided function supersedes the internal \a
   errorFunction function. The error reporting function must be a void function
   with the correct prototype, as shown below.

    In the function prototype
    - The ::RDM_RETCODE is the error code as defined in RDM header files
    - The character pointer points to the error message
    - The ::RDM_DB pointer will be the \a db where the error occurred
    - The void pointer is the same pointer which was passed to
       rdm_dbSetErrorHandler() function, allowing the error reporting
       function to retrieve more information if it is called.

    \code
    void MyErr (
        RDM_RETCODE errnum,
        const RDM_TCHAR_T *errmsg,
        void *userdata,
        RDM_DB db
    );
    \endcode

    \par Locking Requirements
    None

    \retval{sOKAY}

    \see rdm_tfsAllocDatabase
    \see rdm_dbOpen
    \see rdm_dbSetOptions
    \ingroup db_error
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbSetErrorHandler (
    RDM_DB db,                   /**< [in] A valid RDM database handle */
    RDM_ERROR_FCN errorFunction, /**< [in] The custom function that will be
                                 called when RDM detects an error condition */
    void *userData /**< [in] An option pointer to a buffer that will be passed
                   into the error handler when it is called */
);

/** \brief Set a single RDM option from a string

    This function is used to set a single RDM database option from a string
   value. The \a key parameter accepts the key for the option to set.

    \includedoc dox/sqldb-options.mdinc

    \par Locking Requirements
    None

    \retval{sOKAY}
    \retval{eDBOPEN}
    \retval{eINVOPTION}
    \retval{eINVOPTIONVALUE}

    \see rdm_tfsAllocDatabase
    \see rdm_dbOpen
    \see rdm_dbSetOptions
    \see rdm_dbGetOption
    \see rdm_dbSetErrorHandler
    \see rdm_dbFree

    \ingroup db_initialization
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbSetOption (
    RDM_DB db,                  /**< [in] A valid RDM database handle */
    const RDM_TCHAR_T *keyword, /**< [in] Single option keyword */
    const RDM_TCHAR_T *strValue /**< [in] option value in string */
);

/** \brief Set RDM options

    This function is used to set any RDM database options.

    The \a optString is a key value pair in the form of \"\c{key=value}\".

    The options will be inherited from the option values stored in the TFS
   handle when the database handle is created. Changes to the option values
   stored in the TFS handle will not affect those of the database handles that
   have already been allocated except for the \a Timeout option, which changes
   the timeout value of the database that's already opened.

    The options set with this function will not affect the parent TFS handle.

    \par The keys supported are

    \includedoc dox/sqldb-options.mdinc

    \par Locking Requirements
    None.

    \retval{sOKAY}
    \retval{eDBOPEN}
    \retval{eINVOPTION}
    \retval{eINVOPTIONVALUE}

    \see rdm_tfsAllocDatabase
    \see rdm_dbOpen
    \see rdm_dbSetOption
    \see rdm_dbGetOptions
    \see rdm_dbSetErrorHandler
    \see rdm_dbFree
    \see rdm_dbFlushIdIndex
    \see rdm_dbCreateNewPackFile
    \see rdm_dbVacuum

    \ingroup db_initialization
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbSetOptions (
    RDM_DB db,                   /**< [in] A valid RDM database handle */
    const RDM_TCHAR_T *optString /**< [in] The option string */
);

/** \brief Get read locks

    This function will obtain 'read' locks on the specified tables. The
    \a numTableIds parameter will specify how many table locks are being requested.
    The \a tableIds parameter is an array of the identifiers for the tables to
    lock. The \a pTrans parameter can optionally retrieve a handle which will
    allow rdm_transEnd() to commit or rdm_transEndRollback() to rollback
    a transaction to a particular point. In the case of read locks they can be
    freed inside the bounds of an update transaction. If you do not wish read
    locks to be freed inside of an update transaction then you should request
    the read locks when starting the update transaction with rdm_dbStartUpdate().

    \note It is illegal to request locks if there is a database snapshot active for
    the current user.

    \note You can lock all tables by using the special ::RDM_LOCK_ALL \a tableIds
    pointer.

    You can stack calls to rdm_dbStartRead() and rdm_dbStartUpdate().
    Doing so will create transaction marks (if a transaction handle \a pTrans
    parameter is provided) and potentially request additional locks. An update
    transaction will not be committed (or aborted) until the root update
    transaction has been committed/rolled back. This can be done by calling
    rdm_transEnd() or rdm_transEndRollback() with the root ::RDM_TRANS handle.

    If you stack calls to rdm_dbStartRead() or rdm_dbStartUpdate() and
    get a transaction handle (\a pTrans) the transaction handle you receive will
    be invalidated if a transaction handle that was obtained prior is involved
    in a call to rdm_transEnd(), rdm_transEndRollback() or
    rdm_transRollback(). In addition, a call to rdm_dbEnd() or
    rdm_dbEndRollback() will invalidate any existing transaction handles. The
    results of using an invalid transaction handle are undefined.

    A simple use case for reading rows from a table will not use nesting. It
    will get all of the locks upfront with a single call to
    rdm_dbStartRead() and free the locks with a call to rdm_dbEnd().
    In this simple use case there will be no need to use a transaction handle.

    See \ref locking for a more detailed description.

    \par Locking Requirements
    None

    \retval{sOKAY}
    \retval{eDBNOTOPEN}
    \retval{eUNAVAIL}
    \retval{sSCHEMACHANGE}
    \retval{eSNAPSHOTACTIVE}

    \see rdm_dbOpen
    \see rdm_dbGetLockStatus
    \see rdm_cursorGetLockStatus
    \see rdm_dbGetTransactionStatus
    \see rdm_dbStartUpdate
    \see rdm_dbStartSnapshot
    \see rdm_dbGetRows
    \see rdm_cursorMoveToNext
    \see rdm_cursorReadRow
    \see rdm_cursorReadBlob
    \see rdm_cursorReadColumn
    \see rdm_cursorReadKey
    \see rdm_dbEnd
    \see rdm_transEnd

    \ingroup db_locking
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbStartRead (
    RDM_DB db,                    /**< [in] A valid RDM database handle */
    const RDM_TABLE_ID *tableIds, /**< [in] array of table identifiers */
    uint32_t numTableIds, /**< [in] The number of elements in \a tableIds */
    RDM_TRANS *pTrans     /**< [out] An optional transaction handle  */
);

/** \brief Start a snapshot

    This function will start a shapshot of the tables referenced in \a
    tableIds. A snapshot is a static view of the requested tables at the
    time the snapshot is created. Any changes made to those tables will
    not be visibile until the snapshot is released.

    The snapshot will remain in effect until a call to rdm_dbEnd() or a call
    to rdm_transEnd() with the ::RDM_TRANS handle for the snapshot is made.

    \par Locking Requirements
    None

    \retval{sOKAY}
    \retval{eDBNOTOPEN}
    \retval{eTRACTIVE}
    \retval{sSCHEMACHANGE}
    \retval{eSNAPSHOTACTIVE}

    \see rdm_dbOpen
    \see rdm_dbGetLockStatus
    \see rdm_cursorGetLockStatus
    \see rdm_dbGetTransactionStatus
    \see rdm_dbStartRead
    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorMoveToNext
    \see rdm_cursorReadRow
    \see rdm_cursorReadBlob
    \see rdm_cursorReadColumn
    \see rdm_cursorReadKey
    \see rdm_dbEnd
    \see rdm_transEnd

    \ingroup db_locking
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbStartSnapshot (
    RDM_DB db,                    /**< [in] A valid RDM database handle */
    const RDM_TABLE_ID *tableIds, /**< [in] array of table identifiers */
    uint32_t numTableIds, /**< [in] The number of elements in \a tableIds */
    RDM_TRANS *pTrans     /**< [out] optional transaction handle */
);

/** \brief Get write locks

    This function will get 'write' and 'read' locks on the requested tables.
    The \a numWriteTableIds parameter will specify how many write table locks are
    being requested. The \a writeTableIds parameter is an array of the identifiers
    for the tables to write lock. The \a numReadTableIds parameter will specify
    how many read table locks are being requested. The \a readTableIds parameter
    is array of the identifiers for the tables to read lock. The \a pTrans
    parameter can optionally retrieve a handle which will allow
    rdm_transEnd() to commit or rdm_transEndRollback() to rollback a
    transaction to a particular point. In the case of read locks, they can be
    freed inside the bounds of an update transaction.

    The first call to rdm_dbStartUpdate() will begin a root update
    transaction.

    When that root transaction has been commited or rolled back the transaction
    will be completed. If a call to rdm_dbStartUpdate() or
    rdm_dbStartRead() is made while a transaction is already active a
    a transaction mark point will be created (if the \a pTrans parameter is
    specified). It is possible to rollback or commit to a transaction mark
    point, but changes will not be made a part of the database until the root
    transaction has been committed.

    You can lock all tables by using the special ::RDM_LOCK_ALL \a tableIds
    pointer. You can lock no tables by using the special ::RDM_LOCK_NONE
    \a tableId pointer. You can also use a NULL pointer for the \a tableIds
    and a \a numLocks value of 0 to create a transaction mark point
    without locking any additional tables.

    A shema change require a write lock on the schema which can be
    obtained by specifying the special ::RDM_LOCK_SCHEMA \a tableIds pointer to
    \a writeTableIds.  This is a shorthand for a list of table identifiers
    with only ::TABLE_SCHEMA.

    You can stack calls to rdm_dbStartRead() and
    rdm_dbStartUpdate(). Doing so will create transaction marks and
    potentially request additional locks. An update transaction will not be
    committed (or aborted) until the root update transaction has been
    committed/aborted. This can be done by explicitly ending with the root
    ::RDM_TRANS handle or by calling rdm_transEnd() with the root transaction
    or by calling rdm_dbEnd().

    If you stack calls to rdm_dbStartRead() or rdm_dbStartUpdate() and
    get a transaction handle (\a pTrans) the transaction handle you receive will
    be invalidated if a transaction handle that was obtained prior is involved
    in a call to rdm_transEnd(), rdm_transEndRollback(), or
    rdm_transRollback(). In addition a call to rdm_dbEnd() or
    rdm_dbEndRollback() will invalidate any existing transaction handles. The
    results of using an invalided transaction handle are undefined.

    \note It is illegal to request locks if there is a database snapshot active for
    the current user.

    A simple use case for updating rows from a table will not use nesting. It
    will get all of the locks upfront with a single call to
    rdm_dbStartUpdate() and commit the changes with a call to
    rdm_dbEnd(). In this simple use case there will be no need to use a
    transaction handle.

    See \ref locking for a more detailed description.

    \par Locking Requirements
    None.

    \retval{sOKAY}
    \retval{ePRECOMMITTED}
    \retval{eSNAPSHOTACTIVE}
    \retval{eREADONLY}
    \retval{eUNAVAIL}
    \retval{sSCHEMACHANGE}

    \see rdm_dbOpen
    \see rdm_dbGetLockStatus
    \see rdm_cursorGetLockStatus
    \see rdm_dbGetTransactionStatus
    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbGetRows
    \see rdm_cursorMoveToNext
    \see rdm_dbInsertRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_dbAlterCatalog
    \see rdm_cursorReadRow
    \see rdm_cursorReadBlob
    \see rdm_cursorReadColumn
    \see rdm_cursorReadKey
    \see rdm_dbEnd
    \see rdm_dbEndRollback
    \see rdm_transEnd
    \see rdm_transEndRollback

    \ingroup db_locking
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbStartUpdate (
    RDM_DB db,                         /**< [in] A valid RDM database handle */
    const RDM_TABLE_ID *writeTableIds, /**< [in] array of table identifiers */
    uint32_t numWriteTableIds, /**< [in] The number of elements in \a writeTableIds */
    const RDM_TABLE_ID *readTableIds, /**< [in] array of table identifiers */
    uint32_t numReadTableIds, /**< [in] The number of elements in \a readTableIds */
    RDM_TRANS *pTrans         /**< [out] An optional transaction handle */
);

/** \brief Disable use of triggers for an open database

    This function turns off the use of triggers on the opened database
    specified by the ::RDM_DB handle. This allows non-SQL applications
    to make changes to database tables on which triggers are defined
    without executing those triggers.

    \par Locking Requirements
    None.

    \retval{sOKAY}
    \retval{eDBNOTOPEN}

    \see rdm_dbTriggersOn
    \see rdm_dbTriggersStatus

    \ingroup db_initialization
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbTriggersOff (
    RDM_DB db /**< [in] A valid RDM database handle */
);

/** \brief Enable use of triggers for an open database

    This function turns on the use of triggers on the database
    specified by the ::RDM_DB handle. This is intended to be called by
    non-SQL applications so that SQL is linked with the application in
    order to control the execution of triggers.

    This function is located in \lib{rsql}

    \par Locking Requirements
    None.

    \retval{sOKAY}

    \see rdm_tfsAllocDatabase
    \see rdm_dbOpen
    \see rdm_dbTriggersOff
    \see rdm_dbTriggersStatus
    \see rdm_dbSetOptions

    \ingroup db_initialization
    */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbTriggersOn (
    RDM_DB db /**< [in] A valid RDM database handle */
);

/** \brief Check the triggers use status for a database

    This function sets \a pStat to indicate the triggers status
    for the database specified by \a db as follows:

    | Status                   | Description                          |
    |--------------------------|:-------------------------------------|
    | ::RDM_TRIGGERS_UNDEFINED | rdm_dbTriggersOn() not (yet) called. |
    | ::RDM_TRIGGERS_ON        | rdm_dbTriggersOn() has been called.  |
    | ::RDM_TRIGGERS_OFF       | rdm_dbTriggersOff() has been called. |

    \par Locking Requirements
    None.

    \retval{sOKAY}
    \retval{eDBNOTOPEN}

    \see rdm_tfsAllocDatabase
    \see rdm_dbOpen
    \see rdm_dbTriggersOn
    \see rdm_dbTriggersOff
    \see rdm_dbSetOptions

    \ingroup db_initialization
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbTriggersStatus (
    RDM_DB db, /**< [in] A valid RDM database handle */
    RDM_TRIGGERS_STATUS
        *pStat /**< [out] Pointer to triggers status result variable */
);

RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbRepair (
    RDM_DB db,                   /**< [in] A valid RDM database handle */
    const RDM_TCHAR_T *optString /**< [in] A semicolon delimited string that
                                    contains the utility options */
);

/** \brief Populate a buffer with all of the changes to an in-memory database.

    This function will populate a buffer with all of the changes to an in-memory
    database.  The buffer can be used by rdm_dbReplicationReceive to apply
    the changes to another database.

    \par Locking Requirements
    Read lock on all tables

    \retval{sOKAY}
    \retval{sTRUNCATE}

    \see rdm_rdmReplicationReceive

    \ingroup db_replication
    */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbReplicationSend (
    RDM_DB db,               /**< [in] An allocated ::RDM_DB */
    void *replicationBuffer, /**< [in/out] The buffer to write the changes to */
    size_t bufferSize,       /**< [in] The size of replicationBuffer */
    size_t *pBytesWritten /**< [out] The number of bytes written into replicationBuffer */
);

/** \brief Apply a replication buffer with changes to an in-memory database.

    This function will apply a replication buffer containing changes to an
    in-memory database.  The replication buffer is generated by
    rdm_dbReplicationSend.

    \par Locking Requirements
    None.

    \retval{sOKAY}
    \retval{sTRUNCATE}

    \see rdm_rdmReplicationSend

    \ingroup db_replication
    */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dbReplicationReceive (
    RDM_DB db,                     /**< [in] An allocated ::RDM_DB */
    const void *replicationBuffer, /**< [in] The buffer to read the changes from */
    size_t messageSize             /**< [in] The size of the message */
);

#if defined(__cplusplus)
}
#endif
