/* ----------------------------------------------------------------------------
 * Raima Database Manager
 *
 * Copyright (c) 2014 Raima Inc.,  All rights reserved.
 *
 * Use of this software, whether in source code format, or in executable,
 * binary object code form, is governed by the Raima LICENSE which
 * is fully described in the LICENSE.TXT file, included within this
 * distribution of files.
 * ----------------------------------------------------------------------------
 */

/** \file rdmrdmapi.h
 *  \brief Header for the RDM base APIs
 */

#include "rdmtypes.h"
#include "rdmtfstypes.h"

/**
 *
 * \defgroup rdm_rdm RDM static instantiation APIs
 * \ingroup rdm
 *
 *     RDM static instantiation APIs. You may want to start by
 *     instantiation a TFS handle using rdm_rdmAllocTFS(). Please see
 *     the respective sub-modules for details.
 *
 * \addtogroup rdm_rdm
 * @{
 * \defgroup rdm_tfs TFS Instantiation
 *
 *     The TFS instantiation API.  The functions here are located in \lib{tfs}
 *
 * \defgroup rdm_initialization Database Initialization
 *
 *     The core database instantiation API.  The functions here are
 *     located in \lib{tfs}
 *
 * \defgroup rdm_association_other Cursor Association Functions (other)
 *
 *     The core cursor association for the special \e BeforeFirst and
 *     \e AfterLast position.  The functions here are located in
 *     \lib{rdm}
 *
 * \defgroup rdm_encryption Instantiating Encryption Context
 *
 *     The encryption instantiation API.  The functions here are
 *     located in \lib{enc}
 *
 * \defgroup rdm_miscellaneous Miscellaneous Static Functions
 *
 *     Miscelaneous functions API.  The functions here are
 *     located in \lib{rdm}
 *
 * @}*/

#if defined(RDM_UNICODE)
#define rdm_rdmAllocTFS RDM_T_FUNC (rdm_rdmAllocTFS)
#define rdm_rdmAllocTFSWithMemBuf RDM_T_FUNC (rdm_rdmAllocTFSWithMemBuf)
#define rdm_rdmGetVersion RDM_T_FUNC (rdm_rdmGetVersion)
#endif

#if defined(__cplusplus)
extern "C" {
#endif

/** \brief Allocate memory for a new RDM db

    This function allocates memory for and initializes an ::RDM_DB using the
    default ::RDM_TFS. You must call rdm_dbOpen() to associate a database
    with the handle.

    \note Database configuration options associated with the ::RDM_TFS handle
    will be inherited at this point. Changes to the ::RDM_TFS handle options
    after this point will not change the values now associated with the
    ::RDM_DB handle.

    \retval{sOKAY}

    \ingroup rdm_initialization

    \see rdm_tfsAllocDatabase
    \see rdm_dbFree
    \see rdm_dbOpen
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_rdmAllocDatabase (
    RDM_DB *pDb /**< [out] A pointer an RDM_DB */
);

/** \brief Allocate the TFS handle

This function will initialize the TFS service with specified
option values and return its handle. The \a optString is a semicolon-delimited
list of key/value pairs. Each pair is specified in the form of "key=value".
There are two main categories of options that can be set - options for the
::RDM_TFS instance, and database default values.

TFS instance options determine the general behaviour of the ::RDM_TFS handle.
The \ref opt_tfstype option sets the type of the ::RDM_TFS handle. Other options
take effect only when the ::RDM_TFS handle includes an embedded TFS.

\par The TFS instance option keys supported in \a optString are:

\includedoc dox/tfs-options.mdinc

Database configuration options determine the way the TFS instance manages its
databases. These options affect only the ::RDM_TFS handle that includes an
embedded TFS.

The keys supported in \a optString are:

\includedoc dox/sqldb-options.mdinc

\note Database configuration options can also be set using \ref rdm_tfsSetOption
or \ref rdm_tfsSetOptions, in which case those option values will be inherited
by the
::RDM_DB handles allocated off this ::RDM_TFS handle, preceding the default
values stored in the ::RDM_TFS handle.

\retval{sOKAY}
\retval{eTFS_DOCROOTINVALID}
\retval{eTFS_DOCROOTNOTFOUND}
\retval{eTFS_DOCROOTUSED}
\retval{eINVOPTION}
\retval{eINVOPTIONVALUE}
\retval{eNOMEMORY}

\see rdm_rdmAllocTFSWithMemBuf
\see rdm_tfsEnableListener
\see rdm_tfsPing
\see rdm_tfsAllocEncrypt
\see rdm_tfsSetOptions
\see rdm_tfsAllocDatabase
\see rdm_tfsDropDatabase
\see rdm_tfsKillAllRemoteConnections
\see rdm_tfsGetMemUsage
\see rdm_tfsFree

\ingroup rdm_tfs
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_rdmAllocTFS (
    const RDM_TCHAR_T *optString, /**< [in] option string */
    RDM_TFS *phTFS                /**< [out] TFS handle */
);

/** \brief Allocate the TFS with an associated memory buffer

Allocate the TFS handle and use the associated memory buffer
instead of malloc for memory requests.  If no memory buffer is
specified and \a memSize is greater than 0 \a memSize will serve as the
memory limit for the TFS, but memory will be obtained via calls to
malloc.

\note Until further notice the buffer provided and the size
requested will be used for all TFS handles. This function should
only be used once for allocating the first TFS handle.  This
semantic is subject to change.

This function will initialize the TFS service with specified
option values and return its handle. The \a optString is a key/value
pair in the form of "key=value".

\par The keys supported in \a optString are:

\includedoc dox/tfs-options.mdinc

\note Database configuration options can be set using this function and will
be inherited by the database handle at the time it is allocated. For
example, the \c{"durability=consistent"} option can be set using this
function and it will be inherited by the next ::RDM_DB handle allocated
using this ::RDM_TFS handle. See \ref sqldb_options for the list of valid
database configuration option keys.

\retval{sOKAY}
\retval{eTFS_DOCROOTINVALID}
\retval{eTFS_DOCROOTNOTFOUND}
\retval{eTFS_DOCROOTUSED}
\retval{eINVOPTION}
\retval{eINVOPTIONVALUE}
\retval{eNOMEMORY}

\see rdm_rdmAllocTFS
\see rdm_tfsGetMemUsage

\ingroup rdm_tfs
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_rdmAllocTFSWithMemBuf (
    const RDM_TCHAR_T *optString, /**< [in]  option string */
    void *pMemBuf, /**< [in] The memory buffer to use instead of getting memory from malloc */
    size_t memSize, /**< [in] The memory size limit (size of pMemBuf if pMemBuf is not NULL) */
    RDM_TFS *phTFS /**< [out] TFS handle */
);

/** \brief Get the special \e AfterLast cursor

    This function assigns \a pCursor to the special \e AfterLast cursor. The
    \e AfterLast cursor can be used as the second argument to
    rdm_cursorComparePosition(). The cursor is positioned to the \e AfterLast
    position and cannot be changed.

    The special cursors are static and cannot be freed. The \a pCursor parameter
    should not point to an allocated (or associated) ::RDM_CURSOR as you would
    lose its reference.

    \par Locking Requirements
    None

    \retval{sOKAY}

    \see rdm_cursorComparePosition

    \ingroup rdm_association_other
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_rdmGetAfterLast (
    RDM_CURSOR *pCursor /**< [out] A pointer to an ::RDM_CURSOR (this should not
                           point to an allocated or associated cursor) */
);

/** \brief Get the special \e BeforeFirst cursor

    This function assigns \a pCursor to the special \e BeforeFirst cursor. The
    \e BeforeFirst cursor can be used as the second argument to
    rdm_cursorComparePosition(). The cursor is positioned to the
    \e BeforeFirst position and cannot be changed.

    The special cursors are static and cannot be freed. The \a pCursor parameter
    should not point to an allocated (or associated) ::RDM_CURSOR as you would
    lose its reference.

    \par Locking Requirements
    None

    \retval{sOKAY}

    \ingroup rdm_association_other
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_rdmGetBeforeFirst (
    RDM_CURSOR *pCursor /**< [out] A pointer to an ::RDM_CURSOR (this should not
                           point to an allocated or associated cursor) */
);

/** \brief Return RDM Db Engine version information

    This function fills a buffer with a string describing the version of the RDM
    library according to a format string. The format string consists of
    plaintext (copied to the string buffer) and format specifiers introduced by
    a percent ("%") character. See list below.

    If a NULL is passed in for fmt then the default format string will be used.
    The default format string is the full version string "%V". This corresponds
    to "%n %v Build %b"

    rdm_rdmGetVersion() Type Field Characters
    - %n Product Name
    - %v Version number
    - %b Build number
    - %V Full version string
    - %c Copyright date
    - %w Owner name
    - %r Rights text
    - %C Full Copyright string
    - %% Literal percent character

    \retval{sOKAY}

    \par Locking Requirements
    None

    \ingroup rdm_miscellaneous
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_rdmGetVersion (
    const RDM_TCHAR_T *fmt, /**< [in] A string that specifies how the version
                               information is to be formatted. */
    RDM_TCHAR_T *
        buf, /**< [out] A string buffer to receive the formatted version string. */
    size_t bytesIn /**< [in] The length of buf in bytes */
);

#if defined(__cplusplus)
}
#endif
