/* ----------------------------------------------------------------------------
 * Raima Database Manager
 *
 * Copyright (c) 2014 Raima Inc.,  All rights reserved.
 *
 * Use of this software, whether in source code format, or in executable,
 * binary object code form, is governed by the Raima LICENSE which
 * is fully described in the LICENSE.TXT file, included within this
 * distribution of files.
 * ----------------------------------------------------------------------------
 */

/** \file rdmtypes.h
 *  \brief Header for the native RDM Runtime types
 */

#if !defined(RDMTYPES_H_INCLUDED_)
#define RDMTYPES_H_INCLUDED_

#if !defined(RDM_PERL_GENTYPES)
#include "psptypes.h"
#include "rdmretcodetypes.h"

#define RDM_IDENTIFIER_LEN                                                     \
    127 /**< The maximum identifier length measured in characters not including a null terminator */

typedef uint32_t RDM_TABLE_ID;  /**< The RDM table ID */
typedef uint32_t RDM_COLUMN_ID; /**< The RDM column ID */
typedef uint32_t RDM_KEY_ID;    /**< The RDM key ID */
typedef uint32_t RDM_REF_ID;    /**< The RDM foreign reference ID */

/** \brief A catalog with an empty schema
 *
 *  Use this string as a parameter to rdm_dbLoadCatalog() to set an empty schema
 */
#define RDM_CATALOG_EMPTY_SCHEMA             \
    "{\n"                            \
    "  \"database\":{\n"             \
    "    \"catver\":141,\n"          \
    "    \"dbver\":1,\n"             \
    "    \"encryption\":\"none\",\n" \
    "    \"maxdrawers\":55,\n"       \
    "    \"nodrawers\":4,\n"         \
    "    \"nextdrawer\":4,\n"        \
    "    \"nodoms\":0,\n"            \
    "    \"maxdoms\":0,\n"           \
    "    \"maxtabs\":51,\n"          \
    "    \"notabs\":1,\n"            \
    "    \"nexttab\":1,\n"           \
    "    \"drawers\":[\n"            \
    "    ],\n"                       \
    "    \"tables\":[\n"             \
    "    ]\n"                        \
    "  }\n"                          \
    "}\n"

/** \brief The RDM Partial Key data structure */
typedef struct
{
    const void *value;   /**< The key value */
    size_t bytesIn;      /**< The length of value */
    uint16_t numKeyCols; /**< The number of key columns to use for the key.  If
                           this field is zero the function uses all key
                           columns.  If this parameter is a non-zero value only
                           the number of key columns indicated will be used.
                           This parameter is intended for compound keys,
                           although specifying '1' for a non-compound key is
                           safe */
    uint16_t stringLen;  /**< The partial string length (in characters)
                             for the key value.  If this value is non-zero value,
                             only that many characters of the last column being
                             considered (depending on mumKeyCols above) will be
                             used.  The last column must be string type */
} RDM_SEARCH_KEY;

/** \brief Query types for an R-tree index. */
typedef enum
{
    RDM_RTREE_ALL,     /**< Return all entries in the R-tree index */
    RDM_RTREE_EXACT,   /**< Return only an exact match for the query criteria */
    RDM_RTREE_OVERLAP, /**< Return all rows that overlap the query criteria */
    RDM_RTREE_CONTAINS, /**< Return all rows that are contained in the query criteria */
    RDM_RTREE_NEAREST /**< Return the nearest neighbors to the query criteria. */
} RDM_RTREE_TYPE;

/** \brief The RDM R-tree Key data structure */
typedef struct
{
    const void *value;   /**< The key value */
    size_t bytesIn;      /**< The length of value */
    RDM_RTREE_TYPE type; /**< The type of search to perform */
    uint32_t maxNeighbors; /**< For nearest neighbor searches the maximum number to identify */
} RDM_RTREE_KEY;

/** \brief The range include or exclude enumeration */
typedef enum
{
    RDM_RANGE_OPEN = 0, /**< The range includes the specified value */
    RDM_RANGE_CLOSED    /**< The range excludes the specified value */
} RDM_RANGE;

/** \brief The RDM Range Key data structure */
typedef struct
{
    const void *value;   /**< The key value */
    size_t bytesIn;      /**< The length of value */
    uint16_t numKeyCols; /**< The number of key columns to use for the key.  If
                           this field is zero the function uses all key
                           columns.  If this parameter is a non-zero value only
                           the number of key columns indicated will be used.
                           This parameter is intended for compound keys,
                           although specifying '1' for a non-compound key is
                           safe */
    RDM_RANGE range;     /**< RDM_RANGE_OPEN if the range is open or
                              RDM_RANGE_CLOSED if it is closed */
} RDM_RANGE_KEY;

/** \brief The RDM transaction status identifiers */
typedef enum
{
    RDM_TRANS_READ = 1,   /**< A read transaction is active */
    RDM_TRANS_UPDATE,     /**< An update transaction is active */
    RDM_TRANS_INITIALIZE, /**< A database initialization transaction is active */
    RDM_TRANS_SNAPSHOT,   /**< A snapshot is active */
    RDM_TRANS_NONE,       /**< There is no transaction active */
    RDM_TRANS_UNKNOWN     /**< Unknown transaction status */
} RDM_TRANS_STATUS;

/** \brief The RDM user status */
typedef enum
{
    RDM_U_EMPTY = 0, /**< The user does not exist on the TFS */
    RDM_U_LIVE       /**< The user is in a normal state */
} RDM_USER_STATUS;

/** \brief Enumeration for RDM table lock status
 *
 *   This enumeration specifies the table lock status for RDM databases
 */
typedef enum
{
    RDM_LOCK_FREE = 0x0000,      /**< The table is not locked */
    RDM_LOCK_READ = 0x0101,      /**< The table is locked for reads */
    RDM_LOCK_WRITE = 0x0302,     /**< The table is locked for updates */
    RDM_LOCK_SNAPSHOT = 0x0104,  /**< The table has an active snapshot */
    RDM_LOCK_EXCLUSIVE = 0x0308, /**< The database is opened in exclusive mode */
    RDM_LOCK_CATALOG = 0x0310 /**< The database has the catalog write locked */
} RDM_LOCK_STATUS;

typedef uint64_t RDM_TX;

#define RDM_LOCK_NONE                                                          \
    (const RDM_TABLE_ID *) (-1) /**< Don't lock any tables in the database*/
#define RDM_LOCK_ALL                                                           \
    (const RDM_TABLE_ID *) (-2) /**< Lock all of the tables in the database */
#define RDM_LOCK_SCHEMA (const RDM_TABLE_ID *) (-3) /**< Lock the schema */

#define RDM_LOCK_DB_OPEN                                                       \
    (RDM_TABLE_ID) (-4) /**< Lock for opening a database exclusively */
#define TABLE_SCHEMA                                                           \
    (RDM_TABLE_ID) (0x10000) /**< The table ID for the schema */

/** \brief Callback function signature to report status on an ID-index rebuild */
typedef RDM_RETCODE (RDM_REBUILD_INDEX_REPORT_FCN) (
    const char *table,
    const char *indexes,
    uint64_t current,
    uint64_t total);

#define REF_CURSOR                                                             \
    ((RDM_REF_ID) 0) /**< The default foreign reference ID for the cursor */

#define RDM_COL_HAS_VALUE ((RDM_HAS_VALUE_T) 1) /**< Column has a value */
#define RDM_COL_IS_NULL ((RDM_HAS_VALUE_T) 0)   /**< Coulumn is null */
typedef uint8_t
    RDM_HAS_VALUE_T; /**< RDM Type for whether column has a value or is null */

#define RDM_ALL_DATA ((size_t) -1)

/** \brief Macro for getting the size of a column in the struct definition */
#define RDM_COLUMN_SIZE(table, column) sizeof (((table *) 0)->column)

/** \brief Enumeration for RDM cursor positions / statuses
 *
 *   The enumeration identifies the current position for an RDM cursor.
 *   A cursor can be positioned at a row, between rows, before all
 *   rows, or after all rows
 */
typedef enum
{
    CURSOR_NOT_AT_ROW = 0, /**< A cursor not positioned at a valid row. */
    CURSOR_AT_ROW,         /**< A cursor positioned at a valid row. */
    CURSOR_BETWEEN,        /**< A cursor positioned between rows. */
    CURSOR_BEFORE_FIRST,   /**< A cursor positioned before all rows. */
    CURSOR_AFTER_LAST,     /**< A cursor positioned after all rows. */
    CURSOR_DELETED,        /**< A cursor whose current row has been deleted. */
    CURSOR_CHANGED,        /**< A cursor whose key value has been changed. */
    CURSOR_UNLINKED,       /**< A cursor whose curent row has been unlinked. */
    CURSOR_ROW_GONE, /**< A singleton cursor that has had it's row deleted. */
    CURSOR_SET_GONE, /**< A set cursor that has had it's owner deleted. */
    CURSOR_DROPPED /**< A cursor that is based on an object that has been dropped. */
} RDM_CURSOR_STATUS;

/** \brief Enumeration for RDM cursor comparisons
 *
 *   The enumeration allows for position comparisons between two cursors.
 *   A cursor can either be positioned before, after, or equal to the position
 *   of another cursor.  How positioning is determined is based on the type of
 *   cursor being compared.
 */
typedef enum
{
    CURSOR_BEFORE =
        RDM_LT, /**< The cursor is positioned before the position of the other cursor */
    CURSOR_EQUAL = RDM_EQ, /**< The cursor is positioned at the same position as
                              the position of the other cursor */
    CURSOR_AFTER =
        RDM_GT /**< The cursor is positioned after the position of the other cursor */
} RDM_CURSOR_COMPARE;

#endif /* RDM_PERL_GENTYPES */

/** \brief Enumeration for open modes
 *
 *   The enumeration specifies the open modes.  The creat flags always
 *   require a catalog set using rdm_dbSetCatalog().  The append modes
 *   require any catalog that have been set to match the catalog
 *   stored in the database. ::RDM_OPEN_EXCLUSIVE and
 *   ::RDM_OPEN_SHARED default to create if a catalog have been set
 *   and append if it have not been set.
 *
 */
typedef enum
{
    /* GEN_OPEN_MODE_BEGIN */
    RDM_OPEN_SHARED =
        0x0000, /**< Open database in shared mode, requiring locks */
    RDM_OPEN_EXCLUSIVE = 0x0001, /**< Open database in exclusive mode */
    RDM_OPEN_READONLY =
        0x0002, /**< Open database in readonly mode, requiring locks */
    RDM_OPEN_SHARED_APPEND =
        0x0010, /**< Open an existing database in shared mode, requiring locks */
    RDM_OPEN_EXCLUSIVE_APPEND =
        0x0011, /**< Open an existing database in exclusive mode */
    RDM_OPEN_SHARED_CREAT = 0x0020, /**< Open database (create if it does not
                                       exist) in shared mode, requiring locks */
    RDM_OPEN_EXCLUSIVE_CREAT =
        0x0021, /**< Open database (create if it does not exist) in exclusive mode */
    RDM_OPEN_SHARED_CREAT_TRUNC =
        0x0120, /**< Open and create database (truncate if it does exist) in shared mode, requiring locks */
    RDM_OPEN_EXCLUSIVE_CREAT_TRUNC =
        0x0121, /**< Open and create database (truncate if it does exist) in exclusive mode */
    RDM_OPEN_SHARED_CREAT_EXCL =
        0x0220, /**< Create and open database (fail if it does exist) in shared mode, requiring locks */
    RDM_OPEN_EXCLUSIVE_CREAT_EXCL =
        0x0221 /**< Create and open database (fail if it does exist) in exclusive mode */
    /* GEN_OPEN_MODE_END */
} RDM_OPEN_MODE;

#if !defined(RDM_PERL_GENTYPES)

/** \brief Enumeration for cursor types
 *
 *   The enumeration specifies the type of a cursor
 */
typedef enum
{
    CURSOR_TYPE_BEFORE_FIRST =
        1,                  /**< enum value for a BeforeFirst special cursor */
    CURSOR_TYPE_AFTER_LAST, /**< enum value for a AfterLast special cursor */
    CURSOR_TYPE_REC_SCAN,   /**< enum value for a record scan cursor */
    CURSOR_TYPE_KEY_SCAN,   /**< enum value for a key scan cursor */
    CURSOR_TYPE_RTREE_SCAN, /**< enum value for a rtree scan cursor */
    CURSOR_TYPE_SET_SCAN,   /**< enum value for a set scan cursor */
    CURSOR_TYPE_SINGLETON,  /**< enum value for a singleton cursor */
    CURSOR_TYPE_REC_RANGE,  /**< enum value for a rec range cursor */
    CURSOR_TYPE_STATUS,     /**< enum value for a status cursor */
    CURSOR_TYPE_SYSCOLUMN,  /**< enum value for a syscolumn meta cursor */
    CURSOR_TYPE_SYSTABLE,   /**< enum value for a systable meta cursor */
    CURSOR_TYPE_SYSREF,     /**< enum value for a sysref meta cursor */
    CURSOR_TYPE_SYSKEY,     /**< enum value for a syskey meta cursor */
    CURSOR_TYPE_SYSDOMAIN,  /**< enum value for a sysdomain meta cursor */
    CURSOR_TYPE_SYSSORT,    /**< enum value for a syssort meta cursor */
    CURSOR_TYPE_SYSELEM,    /**< enum value for a syselem meta cursor */
    CURSOR_TYPE_SYSDB,      /**< enum value for a sysdb meta cursor */
    CURSOR_TYPE_SYSDRAWER,  /**< enum value for a sysdrawer meta cursor */
    CURSOR_TYPE_ALLOCATED /**< enum value for a cursor that has only been allocated */
} RDM_CURSOR_TYPE;

typedef struct RDM_DB_S *RDM_DB;         /**< RDM database handle */
typedef struct RDM_CURSOR_S *RDM_CURSOR; /**< RDM cursor handle */
typedef struct RDM_TRANS_S *RDM_TRANS;   /**< RDM transaction handle handle */

typedef void(
    EXTERNAL_FCN *RDM_ERROR_FCN) (RDM_DB, RDM_RETCODE, const RDM_TCHAR_T *, void *);
typedef void(
    EXTERNAL_FCN *RDM_ERROR_A_FCN) (RDM_DB, RDM_RETCODE, const char *, void *);
typedef void(
    EXTERNAL_FCN *RDM_ERROR_W_FCN) (RDM_DB, RDM_RETCODE, const wchar_t *, void *);

/** \brief Triggers status values */
typedef enum
{
    RDM_TRIGGERS_ON = 1,       /**< Triggers have been turned on */
    RDM_TRIGGERS_OFF = -1,     /**< Triggers have been turned off */
    RDM_TRIGGERS_UNDEFINED = 0 /**< Triggers status has not been set yet */
} RDM_TRIGGERS_STATUS;

/** \brief RDM Rows status info */
typedef struct
{
    RDM_RETCODE rc; /**< RDM return code */
    uint32_t index; /**< The index for which the return status code applies */
} RDM_ROW_STATUS_INFO;

#endif /* RDM_PERL_GENTYPES */

#endif /* RDMTYPES_H_INCLUDED_ */
